"""CodeContext - Display the block context of code at top of edit window

Once code has scrolled off the top of the screen, it can be difficult
to determine which block you are in.  This extension implements a pane
at the top of each IDLE edit window which provides block structure
hints.  These hints are the lines which contain the block opening
keywords, e.g. 'if', for the enclosing block.  The number of hint lines
is determined by the numlines variable in the CodeContext section of
config-extensions.def. Lines which do not open blocks are not shown in
the context hints pane.

"""
import Tkinter
from configHandler import idleConf
from sets import Set
import re

BLOCKOPENERS = Set(["class", "def", "elif", "else", "except", "finally", "for",
                    "if", "try", "while"])
INFINITY = 1 << 30
UPDATEINTERVAL = 100 # millisec
FONTUPDATEINTERVAL = 1000 # millisec

getspacesfirstword = lambda s, c=re.compile(r"^(\s*)(\w*)"): c.match(s).groups()

class CodeContext:
    menudefs = [('options', [('!Code Conte_xt', '<<toggle-code-context>>')])]

    numlines = idleConf.GetOption("extensions", "CodeContext",
                                  "numlines", type="int", default=3)
    bgcolor = idleConf.GetOption("extensions", "CodeContext",
                                 "bgcolor", type="str", default="LightGray")
    fgcolor = idleConf.GetOption("extensions", "CodeContext",
                                 "fgcolor", type="str", default="Black")
    def __init__(self, editwin):
        self.editwin = editwin
        self.text = editwin.text
        self.textfont = self.text["font"]
        self.label = None
        # Dummy line, which starts the "block" of the whole document:
        self.info = list(self.interesting_lines(1))
        self.lastfirstline = 1
        visible = idleConf.GetOption("extensions", "CodeContext",
                                     "visible", type="bool", default=False)
        if visible:
            self.toggle_code_context_event()
            self.editwin.setvar('<<toggle-code-context>>', True)
        # Start two update cycles, one for context lines, one for font changes.
        self.text.after(UPDATEINTERVAL, self.timer_event)
        self.text.after(FONTUPDATEINTERVAL, self.font_timer_event)

    def toggle_code_context_event(self, event=None):
        if not self.label:
            self.label = Tkinter.Label(self.editwin.top,
                                      text="\n" * (self.numlines - 1),
                                      anchor="w", justify="left",
                                      font=self.textfont,
                                      bg=self.bgcolor, fg=self.fgcolor,
                                      relief="sunken",
                                      width=1, # Don't request more than we get
                                      )
            self.label.pack(side="top", fill="x", expand=0,
                            after=self.editwin.status_bar)
        else:
            self.label.destroy()
            self.label = None
        idleConf.SetOption("extensions", "CodeContext", "visible",
                           str(self.label is not None))
        idleConf.SaveUserCfgFiles()

    def get_line_info(self, linenum):
        """Get the line indent value, text, and any block start keyword

        If the line does not start a block, the keyword value is False.
        The indentation of empty lines (or comment lines) is INFINITY.
        There is a dummy block start, with indentation -1 and text "".

        Return the indent level, text (including leading whitespace),
        and the block opening keyword.

        """
        if linenum == 0:
            return -1, "", True
        text = self.text.get("%d.0" % linenum, "%d.end" % linenum)
        spaces, firstword = getspacesfirstword(text)
        opener = firstword in BLOCKOPENERS and firstword
        if len(text) == len(spaces) or text[len(spaces)] == '#':
            indent = INFINITY
        else:
            indent = len(spaces)
        return indent, text, opener

    def interesting_lines(self, firstline):
        """Generator which yields context lines, starting at firstline."""
        # The indentation level we are currently in:
        lastindent = INFINITY
        # For a line to be interesting, it must begin with a block opening
        # keyword, and have less indentation than lastindent.
        for line_index in xrange(firstline, -1, -1):
            indent, text, opener = self.get_line_info(line_index)
            if indent < lastindent:
                lastindent = indent
                if opener in ("else", "elif"):
                    # We also show the if statement
                    lastindent += 1
                if opener and line_index < firstline:
                    yield line_index, text

    def update_label(self):
        firstline = int(self.text.index("@0,0").split('.')[0])
        if self.lastfirstline == firstline:
            return
        self.lastfirstline = firstline
        tmpstack = []
        for line_index, text in self.interesting_lines(firstline):
            # Remove irrelevant self.info items, and when we reach a relevant
            # item (which must happen because of the dummy element), break.
            while self.info[-1][0] > line_index:
                del self.info[-1]
            if self.info[-1][0] == line_index:
                break
            tmpstack.append((line_index, text))
        while tmpstack:
            self.info.append(tmpstack.pop())
        lines = [""] * max(0, self.numlines - len(self.info)) + \
                [x[1] for x in self.info[-self.numlines:]]
        self.label["text"] = '\n'.join(lines)

    def timer_event(self):
        if self.label:
            self.update_label()
        self.text.after(UPDATEINTERVAL, self.timer_event)

    def font_timer_event(self):
        newtextfont = self.text["font"]
        if self.label and newtextfont != self.textfont:
            self.textfont = newtextfont
            self.label["font"] = self.textfont
        self.text.after(FONTUPDATEINTERVAL, self.font_timer_event)



import W
import sys
from Carbon import Qd

__version__ = "0.2"
__author__ = "jvr"

class _modulebrowser:

    def __init__(self):
        self.editmodules = []
        self.modules = []
        self.window = W.Window((210, 1000), "Module Browser", minsize = (210, 160), maxsize = (340, 20000))

        self.window.openbutton = W.Button((10, 8, 90, 16), "Open", self.openbuttonhit)
        self.window.browsebutton = W.Button((110, 8, 90, 16), "Browse\xc9", self.browsebuttonhit)
        self.window.reloadbutton = W.Button((10, 32, 90, 16), "Reload", self.reloadbuttonhit)
        self.window.openotherbutton = W.Button((110, 32, 90, 16), "Open other\xc9", self.openother)

        self.window.openbutton.enable(0)
        self.window.reloadbutton.enable(0)
        self.window.browsebutton.enable(0)
        self.window.setdefaultbutton(self.window.browsebutton)

        self.window.bind("cmdr", self.window.reloadbutton.push)
        self.window.bind("cmdb", self.window.browsebutton.push)

        self.window.bind("<activate>", self.activate)
        self.window.bind("<close>", self.close)

        self.window.list = W.List((-1, 56, 1, -14), [], self.listhit)

        self.window.open()
        self.checkbuttons()

    def close(self):
        global _browser
        _browser = None

    def activate(self, onoff):
        if onoff:
            self.makelist()

    def listhit(self, isdbl):
        self.checkbuttons()
        if isdbl:
            if self.window._defaultbutton:
                self.window._defaultbutton.push()

    def checkbuttons(self):
        sel = self.window.list.getselection()
        if sel:
            for i in sel:
                if self.editmodules[i]:
                    self.window.openbutton.enable(1)
                    self.window.reloadbutton.enable(1)
                    self.window.setdefaultbutton(self.window.openbutton)
                    break
            else:
                self.window.openbutton.enable(0)
                self.window.reloadbutton.enable(0)
                self.window.setdefaultbutton(self.window.browsebutton)
            self.window.browsebutton.enable(1)
        else:
            #self.window.setdefaultbutton(self.window.browsebutton)
            self.window.openbutton.enable(0)
            self.window.reloadbutton.enable(0)
            self.window.browsebutton.enable(0)

    def openbuttonhit(self):
        import imp
        sel = self.window.list.getselection()
        W.SetCursor("watch")
        for i in sel:
            modname = self.window.list[i]
            try:
                self.openscript(sys.modules[modname].__file__, modname)
            except IOError:
                try:
                    file, path, description = imp.find_module(modname)
                except ImportError:
                    W.SetCursor("arrow")
                    W.Message("Can't find file for module '%s'."
                                    % modname)
                else:
                    self.openscript(path, modname)

    def openscript(self, path, modname):
        import os
        if path[-3:] == '.py':
            W.getapplication().openscript(path, modname=modname)
        elif path[-4:] in ['.pyc', '.pyo']:
            W.getapplication().openscript(path[:-1], modname=modname)
        else:
            W.Message("Can't edit '%s'; it might be a shared library or a .pyc file."
                            % modname)

    def openother(self):
        import imp
        import EasyDialogs

        modname = EasyDialogs.AskString("Open module:")
        if modname:
            try:
                file, path, description = imp.find_module(modname)
            except ImportError:
                if modname in sys.builtin_module_names:
                    alerttext = "'%s' is a builtin module, which you can't edit." % modname
                else:
                    alerttext = "No module named '%s'." % modname
                raise W.AlertError, alerttext
            self.openscript(path, modname)

    def reloadbuttonhit(self):
        sel = self.window.list.getselection()
        W.SetCursor("watch")
        for i in sel:
            mname = self.window.list[i]
            m = sys.modules[mname]
            # Set the __name__ attribute of the module to its real name.
            # reload() complains if it's __main__, which is true
            # when it recently has been run as a script with "Run as __main__"
            # enabled.
            m.__name__ = mname
            reload(m)

    def browsebuttonhit(self):
        sel = self.window.list.getselection()
        if not sel:
            return
        import PyBrowser
        for i in sel:
            PyBrowser.Browser(sys.modules[self.window.list[i]])

    def makelist(self):
        editmodules, modules = getmoduleslist()
        if modules == self.modules:
            return
        self.editmodules, self.modules = editmodules, modules
        self.window.list.setdrawingmode(0)
        sel = self.window.list.getselectedobjects()
        self.window.list.set(self.modules)
        self.window.list.setselectedobjects(sel)
        self.window.list.setdrawingmode(1)


def getmoduleslist():
    import PyBrowser        # for caselesssort function
    moduleitems = sys.modules.items()
    moduleitems = filter(lambda (name, module): module is not None, moduleitems)
    modules = map(lambda (name, module): name, moduleitems)
    modules = PyBrowser.caselesssort(modules)
    editmodules = []
    sysmodules = sys.modules
    modulesappend = editmodules.append
    for m in modules:
        module = sysmodules[m]
        try:
            if sysmodules[m].__file__[-3:] == '.py' or \
                            sysmodules[m].__file__[-4:] in ['.pyc', '.pyo']:
                modulesappend(1)
            else:
                modulesappend(0)
        except AttributeError:
            modulesappend(0)
    return editmodules, modules



_browser = None

def ModuleBrowser():
    global _browser
    if _browser is not None:
        _browser.window.select()
    else:
        _browser = _modulebrowser()



# We can test part of the module without zlib.
try:
    import zlib
except ImportError:
    zlib = None

import zipfile, os, unittest

from StringIO import StringIO
from tempfile import TemporaryFile

from test.test_support import TESTFN, run_unittest

TESTFN2 = TESTFN + "2"

class TestsWithSourceFile(unittest.TestCase):
    def setUp(self):
        line_gen = ("Test of zipfile line %d." % i for i in range(0, 1000))
        self.data = '\n'.join(line_gen)

        # Make a source file with some lines
        fp = open(TESTFN, "wb")
        fp.write(self.data)
        fp.close()

    def zipTest(self, f, compression):
        # Create the ZIP archive
        zipfp = zipfile.ZipFile(f, "w", compression)
        zipfp.write(TESTFN, "another"+os.extsep+"name")
        zipfp.write(TESTFN, TESTFN)
        zipfp.close()

        # Read the ZIP archive
        zipfp = zipfile.ZipFile(f, "r", compression)
        self.assertEqual(zipfp.read(TESTFN), self.data)
        self.assertEqual(zipfp.read("another"+os.extsep+"name"), self.data)
        zipfp.close()

    def testStored(self):
        for f in (TESTFN2, TemporaryFile(), StringIO()):
            self.zipTest(f, zipfile.ZIP_STORED)

    if zlib:
        def testDeflated(self):
            for f in (TESTFN2, TemporaryFile(), StringIO()):
                self.zipTest(f, zipfile.ZIP_DEFLATED)

    def tearDown(self):
        os.remove(TESTFN)
        os.remove(TESTFN2)

class OtherTests(unittest.TestCase):
    def testCloseErroneousFile(self):
        # This test checks that the ZipFile constructor closes the file object
        # it opens if there's an error in the file.  If it doesn't, the traceback
        # holds a reference to the ZipFile object and, indirectly, the file object.
        # On Windows, this causes the os.unlink() call to fail because the
        # underlying file is still open.  This is SF bug #412214.
        #
        fp = open(TESTFN, "w")
        fp.write("this is not a legal zip file\n")
        fp.close()
        try:
            zf = zipfile.ZipFile(TESTFN)
        except zipfile.BadZipfile:
            os.unlink(TESTFN)

    def testNonExistentFileRaisesIOError(self):
        # make sure we don't raise an AttributeError when a partially-constructed
        # ZipFile instance is finalized; this tests for regression on SF tracker
        # bug #403871.

        # The bug we're testing for caused an AttributeError to be raised
        # when a ZipFile instance was created for a file that did not
        # exist; the .fp member was not initialized but was needed by the
        # __del__() method.  Since the AttributeError is in the __del__(),
        # it is ignored, but the user should be sufficiently annoyed by
        # the message on the output that regression will be noticed
        # quickly.
        self.assertRaises(IOError, zipfile.ZipFile, TESTFN)

    def testClosedZipRaisesRuntimeError(self):
        # Verify that testzip() doesn't swallow inappropriate exceptions.
        data = StringIO()
        zipf = zipfile.ZipFile(data, mode="w")
        zipf.writestr("foo.txt", "O, for a Muse of Fire!")
        zipf.close()

        # This is correct; calling .read on a closed ZipFile should throw
        # a RuntimeError, and so should calling .testzip.  An earlier
        # version of .testzip would swallow this exception (and any other)
        # and report that the first file in the archive was corrupt.
        self.assertRaises(RuntimeError, zipf.testzip)

def test_main():
    run_unittest(TestsWithSourceFile, OtherTests)

if __name__ == "__main__":
    test_main()



"""Widgets for the Macintosh. Built on top of FrameWork"""

__version__ = "0.3"

from Wbase import *
from Wcontrols import *
from Wtext import *
from Wlists import *
from Wwindows import *
from Wmenus import *

_application = None
_signature = None

AlertError = 'AlertError'

def setapplication(app, sig):
    global _application, _signature
    _application = app
    _signature = sig

def getapplication():
    if _application is None:
        raise WidgetsError, 'W not properly initialized: unknown Application'
    return _application

def getdefaultfont():
    prefs = getapplication().getprefs()
    if not prefs.defaultfont:
        prefs.defaultfont = ("Geneva", 0, 10, (0, 0, 0))
    return prefs.defaultfont

def Message(text):
    import EasyDialogs, string
    from Carbon import Qd
    Qd.InitCursor()
    text = string.replace(text, "\n", "\r")
    if not text:
        text = '<Alert text not specified>'
    EasyDialogs.Message(text)



#! /usr/bin/env python

# This script is obsolete -- it is kept for historical purposes only.
#
# Fix Python source files to use the new class definition syntax, i.e.,
# the syntax used in Python versions before 0.9.8:
#       class C() = base(), base(), ...: ...
# is changed to the current syntax:
#       class C(base, base, ...): ...
#
# The script uses heuristics to find class definitions that usually
# work but occasionally can fail; carefully check the output!
#
# Command line arguments are files or directories to be processed.
# Directories are searched recursively for files whose name looks
# like a python module.
# Symbolic links are always ignored (except as explicit directory
# arguments).  Of course, the original file is kept as a back-up
# (with a "~" attached to its name).
#
# Changes made are reported to stdout in a diff-like format.
#
# Undoubtedly you can do this using find and sed or perl, but this is
# a nice example of Python code that recurses down a directory tree
# and uses regular expressions.  Also note several subtleties like
# preserving the file's mode and avoiding to even write a temp file
# when no changes are needed for a file.
#
# NB: by changing only the function fixline() you can turn this
# into a program for a different change to Python programs...

import sys
import regex
import os
from stat import *

err = sys.stderr.write
dbg = err
rep = sys.stdout.write

def main():
    bad = 0
    if not sys.argv[1:]: # No arguments
        err('usage: ' + sys.argv[0] + ' file-or-directory ...\n')
        sys.exit(2)
    for arg in sys.argv[1:]:
        if os.path.isdir(arg):
            if recursedown(arg): bad = 1
        elif os.path.islink(arg):
            err(arg + ': will not process symbolic links\n')
            bad = 1
        else:
            if fix(arg): bad = 1
    sys.exit(bad)

ispythonprog = regex.compile('^[a-zA-Z0-9_]+\.py$')
def ispython(name):
    return ispythonprog.match(name) >= 0

def recursedown(dirname):
    dbg('recursedown(%r)\n' % (dirname,))
    bad = 0
    try:
        names = os.listdir(dirname)
    except os.error, msg:
        err('%s: cannot list directory: %r\n' % (dirname, msg))
        return 1
    names.sort()
    subdirs = []
    for name in names:
        if name in (os.curdir, os.pardir): continue
        fullname = os.path.join(dirname, name)
        if os.path.islink(fullname): pass
        elif os.path.isdir(fullname):
            subdirs.append(fullname)
        elif ispython(name):
            if fix(fullname): bad = 1
    for fullname in subdirs:
        if recursedown(fullname): bad = 1
    return bad

def fix(filename):
##  dbg('fix(%r)\n' % (filename,))
    try:
        f = open(filename, 'r')
    except IOError, msg:
        err('%s: cannot open: %r\n' % (filename, msg))
        return 1
    head, tail = os.path.split(filename)
    tempname = os.path.join(head, '@' + tail)
    g = None
    # If we find a match, we rewind the file and start over but
    # now copy everything to a temp file.
    lineno = 0
    while 1:
        line = f.readline()
        if not line: break
        lineno = lineno + 1
        while line[-2:] == '\\\n':
            nextline = f.readline()
            if not nextline: break
            line = line + nextline
            lineno = lineno + 1
        newline = fixline(line)
        if newline != line:
            if g is None:
                try:
                    g = open(tempname, 'w')
                except IOError, msg:
                    f.close()
                    err('%s: cannot create: %r\n' % (tempname, msg))
                    return 1
                f.seek(0)
                lineno = 0
                rep(filename + ':\n')
                continue # restart from the beginning
            rep(repr(lineno) + '\n')
            rep('< ' + line)
            rep('> ' + newline)
        if g is not None:
            g.write(newline)

    # End of file
    f.close()
    if not g: return 0 # No changes

    # Finishing touch -- move files

    # First copy the file's mode to the temp file
    try:
        statbuf = os.stat(filename)
        os.chmod(tempname, statbuf[ST_MODE] & 07777)
    except os.error, msg:
        err('%s: warning: chmod failed (%r)\n' % (tempname, msg))
    # Then make a backup of the original file as filename~
    try:
        os.rename(filename, filename + '~')
    except os.error, msg:
        err('%s: warning: backup failed (%r)\n' % (filename, msg))
    # Now move the temp file to the original file
    try:
        os.rename(tempname, filename)
    except os.error, msg:
        err('%s: rename failed (%r)\n' % (filename, msg))
        return 1
    # Return succes
    return 0

# This expression doesn't catch *all* class definition headers,
# but it's pretty darn close.
classexpr = '^\([ \t]*class +[a-zA-Z0-9_]+\) *( *) *\(\(=.*\)?\):'
classprog = regex.compile(classexpr)

# Expressions for finding base class expressions.
baseexpr = '^ *\(.*\) *( *) *$'
baseprog = regex.compile(baseexpr)

def fixline(line):
    if classprog.match(line) < 0: # No 'class' keyword -- no change
        return line

    (a0, b0), (a1, b1), (a2, b2) = classprog.regs[:3]
    # a0, b0 = Whole match (up to ':')
    # a1, b1 = First subexpression (up to classname)
    # a2, b2 = Second subexpression (=.*)
    head = line[:b1]
    tail = line[b0:] # Unmatched rest of line

    if a2 == b2: # No base classes -- easy case
        return head + ':' + tail

    # Get rid of leading '='
    basepart = line[a2+1:b2]

    # Extract list of base expressions
    bases = basepart.split(',')

    # Strip trailing '()' from each base expression
    for i in range(len(bases)):
        if baseprog.match(bases[i]) >= 0:
            x1, y1 = baseprog.regs[1]
            bases[i] = bases[i][x1:y1]

    # Join the bases back again and build the new line
    basepart = ', '.join(bases)

    return head + '(' + basepart + '):' + tail

if __name__ == '__main__':
    main()



# Prelude to allow running this as a main program
def _init():
    import macresource
    import sys, os
    macresource.need('DITL', 468, "PythonIDE.rsrc")
    widgetrespathsegs = [sys.exec_prefix, "Mac", "Tools", "IDE", "Widgets.rsrc"]
    widgetresfile = os.path.join(*widgetrespathsegs)
    if not os.path.exists(widgetresfile):
        widgetrespathsegs = [os.pardir, "Tools", "IDE", "Widgets.rsrc"]
        widgetresfile = os.path.join(*widgetrespathsegs)
    refno = macresource.need('CURS', 468, widgetresfile)
    if os.environ.has_key('PYTHONIDEPATH'):
        # For development set this environment variable
        ide_path = os.environ['PYTHONIDEPATH']
    elif refno:
        # We're not a fullblown application
        idepathsegs = [sys.exec_prefix, "Mac", "Tools", "IDE"]
        ide_path = os.path.join(*idepathsegs)
        if not os.path.exists(ide_path):
            idepathsegs = [os.pardir, "Tools", "IDE"]
            for p in sys.path:
                ide_path = os.path.join(*([p]+idepathsegs))
                if os.path.exists(ide_path):
                    break

    else:
        # We are a fully frozen application
        ide_path = sys.argv[0]
    if ide_path not in sys.path:
        sys.path.insert(0, ide_path)

if __name__ == '__main__':
    _init()

import W
import Wapplication
from Carbon import Evt
import EasyDialogs
import FrameWork

import sys
import string
import os
import urllib

import pimp

PACKMAN_HOMEPAGE="http://www.python.org/packman"

ELIPSES = '...'

USER_INSTALL_DIR = os.path.join(os.environ.get('HOME', ''),
                                                                'Library',
                                                                'Python',
                                                                sys.version[:3],
                                                                'site-packages')

class PackageManagerMain(Wapplication.Application):

    def __init__(self):
        self.preffilepath = os.path.join("Python", "Package Install Manager Prefs")
        Wapplication.Application.__init__(self, 'Pimp')
        from Carbon import AE
        from Carbon import AppleEvents
        self.defaulturl = ""

        AE.AEInstallEventHandler(AppleEvents.kCoreEventClass, AppleEvents.kAEOpenApplication,
                        self.ignoreevent)
        AE.AEInstallEventHandler(AppleEvents.kCoreEventClass, AppleEvents.kAEReopenApplication,
                        self.ignoreevent)
        AE.AEInstallEventHandler(AppleEvents.kCoreEventClass, AppleEvents.kAEPrintDocuments,
                        self.ignoreevent)
        AE.AEInstallEventHandler(AppleEvents.kCoreEventClass, AppleEvents.kAEQuitApplication,
                        self.quitevent)
        if 1:
            import PyConsole
            # With -D option (OSX command line only) keep stderr, for debugging the IDE
            # itself.
            debug_stderr = None
            if len(sys.argv) >= 2 and sys.argv[1] == '-D':
                debug_stderr = sys.stderr
                del sys.argv[1]
            PyConsole.installoutput()
            if debug_stderr:
                sys.stderr = debug_stderr
        self.domenu_openstandard()
        self.mainloop()

    def makeusermenus(self):
        m = Wapplication.Menu(self.menubar, "File")
        newitem = FrameWork.MenuItem(m, "Open Standard Database", "N", 'openstandard')
        newexpitem = FrameWork.MenuItem(m, "Open Experimental Database", None, 'openexperimental')
        newexpitem.enable(pimp.PIMP_VERSION >= "0.4")
        openitem = FrameWork.MenuItem(m, "Open"+ELIPSES, "O", 'open')
        openURLitem = FrameWork.MenuItem(m, "Open URL"+ELIPSES, "D", 'openURL')
        FrameWork.Separator(m)
        moreinfoitem = FrameWork.MenuItem(m, "More Databases", None, 'opendatabasepage')
        FrameWork.Separator(m)
        closeitem = FrameWork.MenuItem(m, "Close", "W", 'close')
##              saveitem = FrameWork.MenuItem(m, "Save", "S", 'save')
##              saveasitem = FrameWork.MenuItem(m, "Save as"+ELIPSES, None, 'save_as')
##              FrameWork.Separator(m)

        m = Wapplication.Menu(self.menubar, "Edit")
        undoitem = FrameWork.MenuItem(m, "Undo", 'Z', "undo")
        FrameWork.Separator(m)
        cutitem = FrameWork.MenuItem(m, "Cut", 'X', "cut")
        copyitem = FrameWork.MenuItem(m, "Copy", "C", "copy")
        pasteitem = FrameWork.MenuItem(m, "Paste", "V", "paste")
        FrameWork.MenuItem(m, "Clear", None,  "clear")
        FrameWork.Separator(m)
        selallitem = FrameWork.MenuItem(m, "Select all", "A", "selectall")

        m = Wapplication.Menu(self.menubar, "Package")
        runitem = FrameWork.MenuItem(m, "Install", "I", 'install')
        homepageitem = FrameWork.MenuItem(m, "Visit Homepage", None, 'homepage')

        self.openwindowsmenu = Wapplication.Menu(self.menubar, 'Windows')
        self.makeopenwindowsmenu()
        self.makehelpmenu()
        self._menustocheck = [closeitem,
                        undoitem, cutitem, copyitem, pasteitem,
                        selallitem,
                        runitem, homepageitem]

    def makehelpmenu(self):
        python_app = os.path.join(sys.prefix, 'Resources/Python.app')
        help_source = os.path.join(python_app, 'Contents/Resources/English.lproj/Documentation')
        hashelp = os.path.isdir(help_source)

        self.helpmenu = m = self.gethelpmenu()
        helpitem1 = FrameWork.MenuItem(m, "PackageManager Help", None, self.domenu_packmanhelp)
        helpitem1.enable(hashelp)
        helpitem2 = FrameWork.MenuItem(m, "MacPython Help", None, self.domenu_pythonhelp)
        helpitem2.enable(hashelp)

    def quitevent(self, theAppleEvent, theReply):
        self._quit()

    def ignoreevent(self, theAppleEvent, theReply):
        pass

    def opendocsevent(self, theAppleEvent, theReply):
        W.SetCursor('watch')
        import aetools
        parameters, args = aetools.unpackevent(theAppleEvent)
        docs = parameters['----']
        if type(docs) <> type([]):
            docs = [docs]
        for doc in docs:
            fsr, a = doc.FSResolveAlias(None)
            path = fsr.as_pathname()
            path = urllib.pathname2url(path)
            self.opendoc(path)

    def opendoc(self, url):
        if url:
            self.defaulturl = url
        PackageBrowser(url)

    def getabouttext(self):
        return "About Package Manager"+ELIPSES

    def do_about(self, id, item, window, event):
        EasyDialogs.Message("Package Install Manager for Python\nPackMan engine (pimp) version: %s" %
                pimp.PIMP_VERSION)

    def domenu_openstandard(self, *args):
        if pimp.PIMP_VERSION >= "0.4":
            url = pimp.getDefaultDatabase()
        else:
            # 0.3 compatibility
            url = None
        self.opendoc(url)

    def domenu_openexperimental(self, *args):
        database = pimp.getDefaultDatabase(experimental=True)
        self.opendoc(database)

    def domenu_open(self, *args):
        filename = EasyDialogs.AskFileForOpen(typeList=("TEXT",))
        if filename:
            filename = urllib.pathname2url(filename)
            self.opendoc(filename)

    def domenu_openURL(self, *args):
        ok = EasyDialogs.AskYesNoCancel(
                "Warning: by opening a non-standard database "
                "you are trusting the maintainer of it "
                "to run arbitrary code on your machine.",
                yes="OK", no="")
        if ok <= 0: return
        url = EasyDialogs.AskString("URL of database to open:",
                default=self.defaulturl, ok="Open")
        if url:
            self.opendoc(url)

    def domenu_opendatabasepage(self):
        import ic

        icr = ic.IC()
        icr.launchurl(PACKMAN_HOMEPAGE)
    def makeopenwindowsmenu(self):
        for i in range(len(self.openwindowsmenu.items)):
            self.openwindowsmenu.menu.DeleteMenuItem(1)
            self.openwindowsmenu.items = []
        windows = []
        self._openwindows = {}
        for window in self._windows.keys():
            title = window.GetWTitle()
            if not title:
                title = "<no title>"
            windows.append((title, window))
        windows.sort()
        for title, window in windows:
            shortcut = None
            item = FrameWork.MenuItem(self.openwindowsmenu, title, shortcut, callback = self.domenu_openwindows)
            self._openwindows[item.item] = window
        self._openwindowscheckmark = 0
        self.checkopenwindowsmenu()

    def domenu_openwindows(self, id, item, window, event):
        w = self._openwindows[item]
        w.ShowWindow()
        w.SelectWindow()

    def domenu_quit(self):
        self._quit()

    def domenu_save(self, *args):
        print "Save"

    def domenu_pythonhelp(self, *args):
        from Carbon import AH
        AH.AHGotoPage("MacPython Help", None, None)

    def domenu_packmanhelp(self, *args):
        from Carbon import AH
        AH.AHGotoPage("MacPython Help", "packman.html", None)

    def _quit(self):
##              import PyConsole, PyEdit
        for window in self._windows.values():
            try:
                rv = window.close() # ignore any errors while quitting
            except:
                rv = 0   # (otherwise, we can get stuck!)
            if rv and rv > 0:
                return
##              try:
##                      PyConsole.console.writeprefs()
##                      PyConsole.output.writeprefs()
##                      PyEdit.searchengine.writeprefs()
##              except:
##                      # Write to __stderr__ so the msg end up in Console.app and has
##                      # at least _some_ chance of getting read...
##                      # But: this is a workaround for way more serious problems with
##                      # the Python 2.2 Jaguar addon.
##                      sys.__stderr__.write("*** PythonIDE: Can't write preferences ***\n")
        self.quitting = 1

class PimpInterface:

    def setuppimp(self, url):
        self.pimpprefs = pimp.PimpPreferences()
        self.pimpdb = pimp.PimpDatabase(self.pimpprefs)
        if not url:
            url = self.pimpprefs.pimpDatabase
        try:
            self.pimpdb.appendURL(url)
        except IOError, arg:
            rv = "Cannot open %s: %s\n" % (url, arg)
            rv += "\nSee MacPython Package Manager help page."
            return rv
        except:
            rv = "Unspecified error while parsing database: %s\n" % url
            rv += "Usually, this means the database is not correctly formatted.\n"
            rv += "\nSee MacPython Package Manager help page."
            return rv
        # Check whether we can write the installation directory.
        # If not, set to the per-user directory, possibly
        # creating it, if needed.
        installDir = self.pimpprefs.installDir
        if not os.access(installDir, os.R_OK|os.W_OK|os.X_OK):
            rv = self.setuserinstall(1)
            if rv: return rv
        return self.pimpprefs.check()

    def closepimp(self):
        self.pimpdb.close()
        self.pimpprefs = None
        self.pimpdb = None
        self.packages = []

    def setuserinstall(self, onoff):
        rv = ""
        if onoff:
            if not os.path.exists(USER_INSTALL_DIR):
                try:
                    os.makedirs(USER_INSTALL_DIR)
                except OSError, arg:
                    rv = rv + arg + "\n"
            if not USER_INSTALL_DIR in sys.path:
                import site
                reload(site)
            self.pimpprefs.setInstallDir(USER_INSTALL_DIR)
        else:
            self.pimpprefs.setInstallDir(None)
        rv = rv + self.pimpprefs.check()
        return rv

    def getuserinstall(self):
        return self.pimpprefs.installDir == USER_INSTALL_DIR

    def getbrowserdata(self, show_hidden=1):
        packages = self.pimpdb.list()
        if show_hidden:
            self.packages = packages
        else:
            self.packages = []
            for pkg in packages:
                name = pkg.fullname()
                if name[0] == '(' and name[-1] == ')' and not show_hidden:
                    continue
                self.packages.append(pkg)
        rv = []
        for pkg in self.packages:
            name = pkg.fullname()
            status, _ = pkg.installed()
            description = pkg.description()
            description_line1 = description.split('\n')[0]
            rv.append((status, name, description_line1))
        return rv

    def getstatus(self, number):
        pkg = self.packages[number]
        return pkg.installed()

    def installpackage(self, sel, output, recursive, force):
        pkg = self.packages[sel]
        pimpinstaller = pimp.PimpInstaller(self.pimpdb)
        list, messages = pimpinstaller.prepareInstall(pkg, force, recursive)
        if messages:
            return messages
        messages = pimpinstaller.install(list, output)
        return messages

class PackageBrowser(PimpInterface):

    def __init__(self, url = None):
        self.ic = None
        messages = self.setuppimp(url)
        self.setupwidgets()
        self.updatestatus()
        self.showmessages(messages)

    def close(self):
        self.closepimp()

    def setupwidgets(self):
        DESCRIPTION_HEIGHT = 140
        INSTALL_POS = -30
        STATUS_POS = INSTALL_POS - (70 + DESCRIPTION_HEIGHT)
        self.w = W.Window((580, 600), "Python Install Manager", minsize = (400, 400), tabbable = 0)
        self.w.titlebar = W.TextBox((4, 8, 60, 18), 'Packages:')
        self.w.hidden_button = W.CheckBox((-100, 4, 0, 18), 'Show Hidden', self.updatestatus)
        data = self.getbrowserdata()
        self.w.packagebrowser = W.MultiList((4, 24, 0, STATUS_POS-2), data, self.listhit, cols=3)

        self.w.installed_l = W.TextBox((4, STATUS_POS, 70, 12), 'Installed:')
        self.w.installed = W.TextBox((74, STATUS_POS, 0, 12), '')
        self.w.message_l = W.TextBox((4, STATUS_POS+20, 70, 12), 'Status:')
        self.w.message = W.TextBox((74, STATUS_POS+20, 0, 12), '')
        self.w.homepage_button = W.Button((4, STATUS_POS+40, 96, 18), 'View homepage', self.do_homepage)
        self.w.description_l = W.TextBox((4, STATUS_POS+70, 70, 12), 'Description:')
        self.w.description = W.EditText((74, STATUS_POS+70, 0, DESCRIPTION_HEIGHT-4))

        self.w.divline = W.HorizontalLine((0, INSTALL_POS-4, 0, 0))
        self.w.verbose_button = W.CheckBox((84, INSTALL_POS+4, 60, 18), 'Verbose')
        self.w.recursive_button = W.CheckBox((146, INSTALL_POS+4, 120, 18), 'Install dependencies', self.updatestatus)
        self.w.recursive_button.set(1)
        self.w.force_button = W.CheckBox((268, INSTALL_POS+4, 70, 18), 'Overwrite', self.updatestatus)
        self.w.user_button = W.CheckBox((340, INSTALL_POS+4, 140, 18), 'For Current User Only', self.do_user)
        self.w.install_button = W.Button((4, INSTALL_POS+4, 56, 18), 'Install:', self.do_install)
        self.w.open()
        self.w.description.enable(0)

    def updatestatus(self):
        topcell = self.w.packagebrowser.gettopcell()
        sel = self.w.packagebrowser.getselection()
        data = self.getbrowserdata(self.w.hidden_button.get())
        self.w.packagebrowser.setitems(data)
        self.w.user_button.set(self.getuserinstall())
        if len(sel) != 1:
            self.w.installed.set('')
            self.w.message.set('')
            self.w.install_button.enable(0)
            self.w.homepage_button.enable(0)
            self.w.description.set('')
            self.w.verbose_button.enable(0)
            self.w.recursive_button.enable(0)
            self.w.force_button.enable(0)
            self.w.user_button.enable(0)
        else:
            sel = sel[0]
            if sel >= len(self.packages):
                sel = 0
            self.w.packagebrowser.setselection([sel])
            installed, message = self.getstatus(sel)
            self.w.installed.set(installed)
            self.w.message.set(message)
            self.w.install_button.enable(installed != "yes" or self.w.force_button.get())
            self.w.homepage_button.enable(not not self.packages[sel].homepage())
            description = self.packages[sel].description()
            description = description.splitlines()
            description = '\r'.join(description)
            self.w.description.set(description)
            self.w.verbose_button.enable(1)
            self.w.recursive_button.enable(1)
            self.w.force_button.enable(1)
            self.w.user_button.enable(1)
        self.w.packagebrowser.settopcell(topcell)

    def listhit(self, *args, **kwargs):
        self.updatestatus()

    def do_install(self):
        sel = self.w.packagebrowser.getselection()[0]
        if self.w.verbose_button.get():
            output = sys.stdout
        else:
            output = None
        recursive = self.w.recursive_button.get()
        force = self.w.force_button.get()
        messages = self.installpackage(sel, output, recursive, force)

        # Re-read .pth files
        import site
        reload(site)

        self.updatestatus()
        self.showmessages(messages)

    def showmessages(self, messages):
        if messages:
            # To be on the safe side we always show the hidden packages,
            # they may be referred to in the error messages.
            if not self.w.hidden_button.get():
                self.w.hidden_button.set(1)
                self.updatestatus()
            if type(messages) == list:
                messages = '\n'.join(messages)
            if self.w.verbose_button.get():
                sys.stdout.write(messages + '\n')
            EasyDialogs.Message(messages)

    def do_homepage(self):
        sel = self.w.packagebrowser.getselection()[0]
        if not self.ic:
            import ic

            self.ic = ic.IC()
        self.ic.launchurl(self.packages[sel].homepage())

    def do_user(self):
        messages = self.setuserinstall(self.w.user_button.get())
        self.updatestatus()
        self.showmessages(messages)

if __name__ == '__main__':
    PackageManagerMain()



import re
import W
import os
import MacPrefs
import MacOS
import string
import webbrowser
import EasyDialogs


app = W.getapplication()

_titlepat = re.compile('<title>\([^<]*\)</title>')

def sucktitle(path):
    f = open(path)
    text = f.read(1024) # assume the title is in the first 1024 bytes
    f.close()
    lowertext = text.lower()
    matcher = _titlepat.search(lowertext)
    if matcher:
        return matcher.group(1)
    return path

def verifydocpath(docpath):
    try:
        tut = os.path.join(docpath, "tut")
        lib = os.path.join(docpath, "lib")
        ref = os.path.join(docpath, "ref")
        for path in [tut, lib, ref]:
            if not os.path.exists(path):
                return 0
    except:
        return 0
    return 1


_resultscounter = 1

class Results:

    def __init__(self, hits):
        global _resultscounter
        hits = map(lambda (path, hits): (sucktitle(path), path, hits), hits)
        hits.sort()
        self.hits = hits
        nicehits = map(
                        lambda (title, path, hits):
                        title + '\r' + string.join(
                        map(lambda (c, p): "%s (%d)" % (p, c), hits), ', '), hits)
        nicehits.sort()
        self.w = W.Window((440, 300), "Search results %d" % _resultscounter, minsize = (200, 100))
        self.w.results = W.TwoLineList((-1, -1, 1, -14), nicehits, self.listhit)
        self.w.open()
        self.w.bind('return', self.listhit)
        self.w.bind('enter', self.listhit)
        _resultscounter = _resultscounter + 1

    def listhit(self, isdbl = 1):
        if isdbl:
            for i in self.w.results.getselection():
                path = self.hits[i][1]
                url = "file://" + "/".join(path.split(":"))
                webbrowser.open(url)


class Status:

    def __init__(self):
        self.w = W.Dialog((440, 64), "Searching\xc9")
        self.w.searching = W.TextBox((4, 4, -4, 16), "")
        self.w.hits = W.TextBox((4, 24, -4, 16), "Hits: 0")
        self.w.canceltip = W.TextBox((4, 44, -4, 16), "Type cmd-period (.) to cancel.")
        self.w.open()

    def set(self, path, hits):
        self.w.searching.set(path)
        self.w.hits.set('Hits: %r' % (hits,))
        app.breathe()

    def close(self):
        self.w.close()


def match(text, patterns, all):
    hits = []
    hitsappend = hits.append
    stringcount = string.count
    for pat in patterns:
        c = stringcount(text, pat)
        if c > 0:
            hitsappend((c, pat))
        elif all:
            hits[:] = []
            break
    hits.sort()
    hits.reverse()
    return hits


def dosearch(docpath, searchstring, settings):
    (docpath, kind, case, word, tut, lib, ref, ext, api) = settings
    books = [(tut, 'tut'), (lib, 'lib'), (ref, 'ref'), (ext, 'ext'), (api, 'api')]
    if not case:
        searchstring = string.lower(searchstring)

    if kind == 1:
        patterns = string.split(searchstring)
        all = 1
    elif kind == 2:
        patterns = string.split(searchstring)
        all = 0
    else:
        patterns = [searchstring]
        all = 0 # not relevant

    ospathjoin = os.path.join
    stringlower = string.lower
    status = Status()
    statusset = status.set
    _match = match
    _open = open
    hits = {}
    try:
        if hasattr(MacOS, 'EnableAppswitch'):
            MacOS.EnableAppswitch(0)
        try:
            for do, name in books:
                if not do:
                    continue
                bookpath = ospathjoin(docpath, name)
                if not os.path.exists(bookpath):
                    continue
                files = os.listdir(bookpath)
                for file in files:
                    fullpath = ospathjoin(bookpath, file)
                    if fullpath[-5:] <> '.html':
                        continue
                    statusset(fullpath, len(hits))
                    f = _open(fullpath)
                    text = f.read()
                    if not case:
                        text = stringlower(text)
                    f.close()
                    filehits = _match(text, patterns, all)
                    if filehits:
                        hits[fullpath] = filehits
        finally:
            if hasattr(MacOS, 'EnableAppswitch'):
                MacOS.EnableAppswitch(-1)
            status.close()
    except KeyboardInterrupt:
        pass
    hits = hits.items()
    hits.sort()
    return hits


class PyDocSearch:

    def __init__(self):
        prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
        try:
            (docpath, kind, case, word, tut, lib, ref, ext, api) = prefs.docsearchengine
        except:
            (docpath, kind, case, word, tut, lib, ref, ext, api) = prefs.docsearchengine = \
                    ("", 0, 0, 0, 1, 1, 0, 0, 0)

        if docpath and not verifydocpath(docpath):
            docpath = ""

        self.w = W.Window((400, 200), "Search the Python Documentation")
        self.w.searchtext = W.EditText((10, 10, -100, 20), callback = self.checkbuttons)
        self.w.searchbutton = W.Button((-90, 12, 80, 16), "Search", self.search)
        buttons = []

        gutter = 10
        width = 130
        bookstart = width + 2 * gutter
        self.w.phraseradio = W.RadioButton((10, 38, width, 16), "As a phrase", buttons)
        self.w.allwordsradio = W.RadioButton((10, 58, width, 16), "All words", buttons)
        self.w.anywordsradio = W.RadioButton((10, 78, width, 16), "Any word", buttons)
        self.w.casesens = W.CheckBox((10, 98, width, 16), "Case sensitive")
        self.w.wholewords = W.CheckBox((10, 118, width, 16), "Whole words")
        self.w.tutorial = W.CheckBox((bookstart, 38, -10, 16), "Tutorial")
        self.w.library = W.CheckBox((bookstart, 58, -10, 16), "Library reference")
        self.w.langueref = W.CheckBox((bookstart, 78, -10, 16), "Lanuage reference manual")
        self.w.extending = W.CheckBox((bookstart, 98, -10, 16), "Extending & embedding")
        self.w.api = W.CheckBox((bookstart, 118, -10, 16), "C/C++ API")

        self.w.setdocfolderbutton = W.Button((10, -30, 100, 16), "Set doc folder", self.setdocpath)

        if docpath:
            self.w.setdefaultbutton(self.w.searchbutton)
        else:
            self.w.setdefaultbutton(self.w.setdocfolderbutton)

        self.docpath = docpath
        if not docpath:
            docpath = "(please select the Python html documentation folder)"
        self.w.docfolder = W.TextBox((120, -28, -10, 16), docpath)

        [self.w.phraseradio, self.w.allwordsradio, self.w.anywordsradio][kind].set(1)

        self.w.casesens.set(case)
        self.w.wholewords.set(word)
        self.w.tutorial.set(tut)
        self.w.library.set(lib)
        self.w.langueref.set(ref)
        self.w.extending.set(ext)
        self.w.api.set(api)

        self.w.open()
        self.w.wholewords.enable(0)
        self.w.bind('<close>', self.close)
        self.w.searchbutton.enable(0)

    def search(self):
        hits = dosearch(self.docpath, self.w.searchtext.get(), self.getsettings())
        if hits:
            Results(hits)
        elif hasattr(MacOS, 'SysBeep'):
            MacOS.SysBeep(0)

    def setdocpath(self):
        docpath = EasyDialogs.AskFolder()
        if docpath:
            if not verifydocpath(docpath):
                W.Message("This does not seem to be a Python documentation folder...")
            else:
                self.docpath = docpath
                self.w.docfolder.set(docpath)
                self.w.setdefaultbutton(self.w.searchbutton)

    def close(self):
        prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
        prefs.docsearchengine = self.getsettings()

    def getsettings(self):
        radiobuttons = [self.w.phraseradio, self.w.allwordsradio, self.w.anywordsradio]
        for i in range(3):
            if radiobuttons[i].get():
                kind = i
                break
        docpath = self.docpath
        case = self.w.casesens.get()
        word = self.w.wholewords.get()
        tut = self.w.tutorial.get()
        lib = self.w.library.get()
        ref = self.w.langueref.get()
        ext = self.w.extending.get()
        api = self.w.api.get()
        return (docpath, kind, case, word, tut, lib, ref, ext, api)

    def checkbuttons(self):
        self.w.searchbutton.enable(not not self.w.searchtext.get())



from Carbon.Res import *
from Carbon.Resources import *
import MacOS

READ = 1
WRITE = 2
smAllScripts = -3

def copyres(src, dst):
    """Copy resource from src file to dst file."""

    cur = CurResFile()
    ctor, type = MacOS.GetCreatorAndType(src)
    input = FSpOpenResFile(src, READ)
    try:
        FSpCreateResFile(dst, ctor, type, smAllScripts)
    except:
        raw_input("%s already exists...  CR to write anyway! " % dst)
    output = FSpOpenResFile(dst, WRITE)
    UseResFile(input)
    ntypes = Count1Types()
    for itype in range(1, 1+ntypes):
        type = Get1IndType(itype)
        nresources = Count1Resources(type)
        for ires in range(1, 1+nresources):
            res = Get1IndResource(type, ires)
            res.LoadResource()
            id, type, name = res.GetResInfo()
            size = res.SizeResource()
            attrs = res.GetResAttrs()
            print id, type, name, size, hex(attrs)
            res.DetachResource()
            UseResFile(output)
            try:
                res2 = Get1Resource(type, id)
            except (RuntimeError, Res.Error), msg:
                res2 = None
            if res2:
                print "Duplicate type+id, not copied"
                print (res2.size, res2.data)
                print res2.GetResInfo()
                if res2.HomeResFile() == output:
                    'OK'
                elif res2.HomeResFile() == input:
                    'BAD!'
                else:
                    print 'Home:', res2.HomeResFile()
            else:
                res.AddResource(type, id, name)
                #res.SetResAttrs(attrs)
                res.WriteResource()
            UseResFile(input)
    UseResFile(cur)
    CloseResFile(output)
    CloseResFile(input)

copyres('::python.¹.rsrc', '::foo.rsrc')



# Scan the tree passed as argv[0] for .rsrc files, skipping .rsrc.df.rsrc
# files, and open these. The effect of this is to create the .rsrc.df.rsrc
# cache files if needed.
# These are needed on OSX: the .rsrc files are in reality AppleSingle-encoded
# files. We decode the resources into a datafork-based resource file.

import macresource
import os
import sys
import getopt

class NoArgsError(Exception):
    pass

def handler((verbose, force), dirname, fnames):
    for fn in fnames:
        if fn[-5:] == '.rsrc' and fn[-13:] != '.rsrc.df.rsrc':
            if force:
                try:
                    os.unlink(os.path.join(dirname, fn + '.df.rsrc'))
                except IOError:
                    pass
            macresource.open_pathname(os.path.join(dirname, fn), verbose=verbose)

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'vf')
        if not args:
            raise NoArgsError
    except (getopt.GetoptError, NoArgsError):
        sys.stderr.write('Usage: cachersrc.py dirname ...\n')
        sys.exit(1)
    verbose = 0
    force = 0
    for o, v in opts:
        if o == '-v':
            verbose = 1
        if o == '-f':
            force = 1
    for dir in sys.argv[1:]:
        os.path.walk(dir, handler, (verbose, force))

if __name__ == '__main__':
    main()



"""Suite Required suite:
Level 0, version 0

Generated from /Volumes/Sap/Applications (Mac OS 9)/Netscape Communicator\xe2\x84\xa2 Folder/Netscape Communicator\xe2\x84\xa2
AETE/AEUT resource version 1/0, language 0, script 0
"""

import aetools
import MacOS

_code = 'reqd'

from StdSuites.Required_Suite import *
class Required_suite_Events(Required_Suite_Events):

    def open(self, _object, _attributes={}, **_arguments):
        """open: Open the specified object(s)
        Required argument: list of objects to open
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'aevt'
        _subcode = 'odoc'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def print_(self, _object, _attributes={}, **_arguments):
        """print: Print the specified object(s)
        Required argument: list of objects to print
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'aevt'
        _subcode = 'pdoc'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def quit(self, _no_object=None, _attributes={}, **_arguments):
        """quit: Quit Navigator
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'aevt'
        _subcode = 'quit'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object != None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def run(self, _no_object=None, _attributes={}, **_arguments):
        """run: Sent to an application when it is double-clicked
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'aevt'
        _subcode = 'oapp'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object != None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']


#
# Indices of types declared in this module
#
_classdeclarations = {
}

_propdeclarations = {
}

_compdeclarations = {
}

_enumdeclarations = {
}



import W
import Wkeys
import struct
import string
import types
import re
from Carbon import Qd, Icn, Fm, QuickDraw
from Carbon.QuickDraw import hilitetransfermode


nullid = '\0\0'
closedid = struct.pack('h', 468)
openid = struct.pack('h', 469)
closedsolidid = struct.pack('h', 470)
opensolidid = struct.pack('h', 471)

arrows = (nullid, closedid, openid, closedsolidid, opensolidid)

has_ctlcharsRE = re.compile(r'[\000-\037\177-\377]')
def ctlcharsREsearch(str):
    if has_ctlcharsRE.search(str) is None:
        return -1
    return 1

def double_repr(key, value, truncvalue = 0,
                        type = type, StringType = types.StringType,
                        has_ctlchars = ctlcharsREsearch, _repr = repr, str = str):
    if type(key) == StringType and has_ctlchars(key) < 0:
        key = str(key)
    else:
        key = _repr(key)
    if key == '__builtins__':
        value = "<" + type(value).__name__ + " '__builtin__'>"
    elif key == '__return__':
        # bleh, when returning from a class codeblock we get infinite recursion in repr.
        # Use safe repr instead.
        import repr
        value = repr.repr(value)
    else:
        try:
            value = _repr(value)
            '' + value      # test to see if it is a string, in case a __repr__ method is buggy
        except:
            value = '\xa5\xa5\xa5 exception in repr()'
    if truncvalue:
        return key + '\t' + value[:255]
    return key + '\t' + value


def truncString(s, maxwid):
    if maxwid < 1:
        return 1, ""
    strlen = len(s)
    strwid = Qd.TextWidth(s, 0, strlen);
    if strwid <= maxwid:
        return 0, s

    Qd.TextFace(QuickDraw.condense)
    strwid = Qd.TextWidth(s, 0, strlen)
    ellipsis = Qd.StringWidth('\xc9')

    if strwid <= maxwid:
        Qd.TextFace(0)
        return 1, s
    if strwid < 1:
        Qd.TextFace(0)
        return 1, ""

    mid = int(strlen * maxwid / strwid)
    while 1:
        if mid <= 0:
            mid = 0
            break
        strwid = Qd.TextWidth(s, 0, mid) + ellipsis
        strwid2 = Qd.TextWidth(s, 0, mid + 1) + ellipsis
        if strwid <= maxwid and maxwid <= strwid2:
            if maxwid == strwid2:
                mid += 1
            break
        if strwid > maxwid:
            mid -= 1
            if mid <= 0:
                mid = 0
                break
        elif strwid2 < maxwid:
            mid += 1
    Qd.TextFace(0)
    return 1, s[:mid] + '\xc9'


def drawTextCell(text, cellRect, ascent, theList):
    l, t, r, b = cellRect
    cellwidth = r - l
    Qd.MoveTo(int(l + 2), int(t + ascent))
    condense, text = truncString(text, cellwidth - 3)
    if condense:
        Qd.TextFace(QuickDraw.condense)
    Qd.DrawText(text, 0, len(text))
    Qd.TextFace(0)


PICTWIDTH = 16


class BrowserWidget(W.CustomList):

    def __init__(self, possize, object = None, col = 100, closechildren = 0):
        W.List.__init__(self, possize, callback = self.listhit)
        self.object = (None,)
        self.indent = 16
        self.lastmaxindent = 0
        self.closechildren = closechildren
        self.children = []
        self.mincol = 64
        self.setcolumn(col)
        self.bind('return', self.openselection)
        self.bind('enter', self.openselection)
        if object is not None:
            self.set(object)

    def set(self, object):
        if self.object[0] is not object:
            self.object = object,
            self[:] = self.unpack(object, 0)
        elif self._parentwindow is not None and self._parentwindow.wid:
            self.update()

    def unpack(self, object, indent):
        return unpack_object(object, indent)

    def update(self):
        # for now...
        W.SetCursor('watch')
        self.setdrawingmode(0)
        sel = self.getselectedobjects()
        fold = self.getunfoldedobjects()
        topcell = self.gettopcell()
        self[:] = self.unpack(self.object[0], 0)
        self.unfoldobjects(fold)
        self.setselectedobjects(sel)
        self.settopcell(topcell)
        self.setdrawingmode(1)

    def setcolumn(self, col):
        self.col = col
        self.colstr = struct.pack('h', col)
        if self._list:
            sel = self.getselection()
            self.setitems(self.items)
            self.setselection(sel)

    def key(self, char, event):
        if char in (Wkeys.leftarrowkey, Wkeys.rightarrowkey):
            sel = self.getselection()
            sel.reverse()
            self.setdrawingmode(0)
            for index in sel:
                self.fold(index, char == Wkeys.rightarrowkey)
            self.setdrawingmode(1)
        else:
            W.List.key(self, char, event)

    def rollover(self, (x, y), onoff):
        if onoff:
            if self.incolumn((x, y)):
                W.SetCursor('hmover')
            else:
                W.SetCursor('arrow')

    def inarrow(self, (x, y)):
        cl, ct, cr, cb = self._list.LRect((0, 0))
        l, t, r, b = self._bounds
        if (x - cl) < 16:
            cellheight = cb - ct
            index = (y - ct) / cellheight
            if index < len(self.items):
                return 1, index
        return None, None

    def incolumn(self, (x, y)):
        l, t, r, b = self._list.LRect((0, 0))
        abscol = l + self.col
        return abs(abscol - x) < 3

    def trackcolumn(self, (x, y)):
        from Carbon import Qd, QuickDraw, Evt
        self.SetPort()
        l, t, r, b = self._bounds
        bounds = l, t, r, b = l + 1, t + 1, r - 16, b - 1
        abscol = l + self.col
        mincol = l + self.mincol
        maxcol = r - 10
        diff = abscol - x
        Qd.PenPat('\000\377\000\377\000\377\000\377')
        Qd.PenMode(QuickDraw.srcXor)
        rect = abscol - 1, t, abscol, b
        Qd.PaintRect(rect)
        lastpoint = (x, y)
        newcol = -1
        #W.SetCursor('fist')
        while Evt.Button():
            Evt.WaitNextEvent(0, 1, None)  # needed for OSX
            (x, y) = Evt.GetMouse()
            if (x, y) <> lastpoint:
                newcol = x + diff
                newcol = max(newcol, mincol)
                newcol = min(newcol, maxcol)
                Qd.PaintRect(rect)
                rect = newcol - 1, t, newcol, b
                Qd.PaintRect(rect)
                lastpoint = (x, y)
        Qd.PaintRect(rect)
        Qd.PenPat(Qd.GetQDGlobalsBlack())
        Qd.PenNormal()
        if newcol > 0 and newcol <> abscol:
            self.setcolumn(newcol - l)

    def click(self, point, modifiers):
        if point == (-1, -1):   # gross.
            W.List.click(self, point ,modifiers)
            return
        hit, index = self.inarrow(point)
        if hit:
            (key, value, arrow, indent) = self.items[index]
            self.fold(index, arrow == 1)
        elif self.incolumn(point):
            self.trackcolumn(point)
        else:
            W.List.click(self, point, modifiers)

    # for W.List.key
    def findmatch(self, tag):
        lower = string.lower
        items = self.items
        taglen = len(tag)
        match = '\377' * 100
        match_i = -1
        for i in range(len(items)):
            item = lower(str(items[i][0]))
            if tag <= item < match:
                match = item
                match_i = i
        if match_i >= 0:
            return match_i
        else:
            return len(items) - 1

    def close(self):
        if self.closechildren:
            for window in self.children:
                window.close()
        self.children = []
        W.List.close(self)

    def fold(self, index, onoff):
        (key, value, arrow, indent) = self.items[index]
        if arrow == 0 or (onoff and arrow == 2) or (not onoff and arrow == 1):
            return
        W.SetCursor('watch')
        topcell = self.gettopcell()
        if onoff:
            self[index] = (key, value, 4, indent)
            self.setdrawingmode(0)
            self[index+1:index+1] = self.unpack(value, indent + 1)
            self[index] = (key, value, 2, indent)
        else:
            self[index] = (key, value, 3, indent)
            self.setdrawingmode(0)
            count = 0
            for i in range(index + 1, len(self.items)):
                (dummy, dummy, dummy, subindent) = self.items[i]
                if subindent <= indent:
                    break
                count = count + 1
            self[index+1:index+1+count] = []
            self[index] = (key, value, 1, indent)
        maxindent = self.getmaxindent()
        if maxindent <> self.lastmaxindent:
            newabsindent = self.col + (maxindent - self.lastmaxindent) * self.indent
            if newabsindent >= self.mincol:
                self.setcolumn(newabsindent)
            self.lastmaxindent = maxindent
        self.settopcell(topcell)
        self.setdrawingmode(1)

    def unfoldobjects(self, objects):
        for obj in objects:
            try:
                index = self.items.index(obj)
            except ValueError:
                pass
            else:
                self.fold(index, 1)

    def getunfoldedobjects(self):
        curindent = 0
        objects = []
        for index in range(len(self.items)):
            (key, value, arrow, indent) = self.items[index]
            if indent > curindent:
                (k, v, a, i) = self.items[index - 1]
                objects.append((k, v, 1, i))
                curindent = indent
            elif indent < curindent:
                curindent = indent
        return objects

    def listhit(self, isdbl):
        if isdbl:
            self.openselection()

    def openselection(self):
        import os
        sel = self.getselection()
        for index in sel:
            (key, value, arrow, indent) = self[index]
            if arrow:
                self.children.append(Browser(value))
            elif type(value) == types.StringType and '\0' not in value:
                editor = self._parentwindow.parent.getscript(value)
                if editor:
                    editor.select()
                    return
                elif os.path.exists(value) and os.path.isfile(value):
                    if MacOS.GetCreatorAndType(value)[1] in ('TEXT', '\0\0\0\0'):
                        W.getapplication().openscript(value)

    def itemrepr(self, (key, value, arrow, indent), str = str, double_repr = double_repr,
                    arrows = arrows, pack = struct.pack):
        arrow = arrows[arrow]
        return arrow + pack('h', self.indent * indent) + self.colstr + \
                        double_repr(key, value, 1)

    def getmaxindent(self, max = max):
        maxindent = 0
        for item in self.items:
            maxindent = max(maxindent, item[3])
        return maxindent

    def domenu_copy(self, *args):
        sel = self.getselectedobjects()
        selitems = []
        for key, value, dummy, dummy in sel:
            selitems.append(double_repr(key, value))
        text = string.join(selitems, '\r')
        if text:
            from Carbon import Scrap
            if hasattr(Scrap, 'PutScrap'):
                Scrap.ZeroScrap()
                Scrap.PutScrap('TEXT', text)
            else:
                Scrap.ClearCurrentScrap()
                sc = Scrap.GetCurrentScrap()
                sc.PutScrapFlavor('TEXT', 0, text)

    def listDefDraw(self, selected, cellRect, theCell,
                    dataOffset, dataLen, theList):
        self.myDrawCell(0, selected, cellRect, theCell,
                dataOffset, dataLen, theList)

    def listDefHighlight(self, selected, cellRect, theCell,
                    dataOffset, dataLen, theList):
        self.myDrawCell(1, selected, cellRect, theCell,
                dataOffset, dataLen, theList)

    def myDrawCell(self, onlyHilite, selected, cellRect, theCell,
                    dataOffset, dataLen, theList):
        savedPort = Qd.GetPort()
        Qd.SetPort(theList.GetListPort())
        savedClip = Qd.NewRgn()
        Qd.GetClip(savedClip)
        Qd.ClipRect(cellRect)
        savedPenState = Qd.GetPenState()
        Qd.PenNormal()

        l, t, r, b = cellRect

        if not onlyHilite:
            Qd.EraseRect(cellRect)

            ascent, descent, leading, size, hm = Fm.FontMetrics()
            linefeed = ascent + descent + leading

            if dataLen >= 6:
                data = theList.LGetCell(dataLen, theCell)
                iconId, indent, tab = struct.unpack("hhh", data[:6])
                try:
                    key, value = data[6:].split("\t", 1)
                except ValueError:
                    # bogus data, at least don't crash.
                    indent = 0
                    tab = 0
                    iconId = 0
                    key = ""
                    value = data[6:]

                if iconId:
                    try:
                        theIcon = Icn.GetCIcon(iconId)
                    except Icn.Error:
                        pass
                    else:
                        rect = (0, 0, 16, 16)
                        rect = Qd.OffsetRect(rect, l, t)
                        rect = Qd.OffsetRect(rect, 0, (theList.cellSize[1] - (rect[3] - rect[1])) / 2)
                        Icn.PlotCIcon(rect, theIcon)

                if len(key) >= 0:
                    cl, ct, cr, cb = cellRect
                    vl, vt, vr, vb = self._viewbounds
                    cl = vl + PICTWIDTH + indent
                    cr = vl + tab
                    if cr > vr:
                        cr = vr
                    if cl < cr:
                        drawTextCell(key, (cl, ct, cr, cb), ascent, theList)
                    cl = vl + tab
                    cr = vr
                    if cl < cr:
                        drawTextCell(value, (cl, ct, cr, cb), ascent, theList)
            #elif dataLen != 0:
            #       drawTextCell("???", 3, cellRect, ascent, theList)
            else:
                return  # we have bogus data

            # draw nice dotted line
            l, t, r, b = cellRect
            l = self._viewbounds[0] + tab
            r = l + 1;
            if not (theList.cellSize[1] & 0x01) or (t & 0x01):
                myPat = "\xff\x00\xff\x00\xff\x00\xff\x00"
            else:
                myPat = "\x00\xff\x00\xff\x00\xff\x00\xff"
            Qd.PenPat(myPat)
            Qd.PenMode(QuickDraw.srcCopy)
            Qd.PaintRect((l, t, r, b))
            Qd.PenNormal()

        if selected or onlyHilite:
            l, t, r, b = cellRect
            l = self._viewbounds[0] + PICTWIDTH
            r = self._viewbounds[2]
            Qd.PenMode(hilitetransfermode)
            Qd.PaintRect((l, t, r, b))

        # restore graphics environment
        Qd.SetPort(savedPort)
        Qd.SetClip(savedClip)
        Qd.DisposeRgn(savedClip)
        Qd.SetPenState(savedPenState)



class Browser:

    def __init__(self, object = None, title = None, closechildren = 0):
        if hasattr(object, '__name__'):
            name = object.__name__
        else:
            name = ''
        if title is None:
            title = 'Object browser'
            if name:
                title = title + ': ' + name
        self.w = w = W.Window((300, 400), title, minsize = (100, 100))
        w.info = W.TextBox((18, 8, -70, 15))
        w.updatebutton = W.BevelButton((-64, 4, 50, 16), 'Update', self.update)
        w.browser = BrowserWidget((-1, 24, 1, -14), None)
        w.bind('cmdu', w.updatebutton.push)
        w.open()
        self.set(object, name)

    def close(self):
        if self.w.wid:
            self.w.close()

    def set(self, object, name = ''):
        W.SetCursor('watch')
        tp = type(object).__name__
        try:
            length = len(object)
        except:
            length = -1
        if not name and hasattr(object, '__name__'):
            name = object.__name__
        if name:
            info = name + ': ' + tp
        else:
            info = tp
        if length >= 0:
            if length == 1:
                info = info + ' (%d element)' % length
            else:
                info = info + ' (%d elements)' % length
        self.w.info.set(info)
        self.w.browser.set(object)

    def update(self):
        self.w.browser.update()


SIMPLE_TYPES = (
        type(None),
        int,
        long,
        float,
        complex,
        str,
        unicode,
)

def get_ivars(obj):
    """Return a list the names of all (potential) instance variables."""
    # __mro__ recipe from Guido
    slots = {}
    # old-style C objects
    if hasattr(obj, "__members__"):
        for name in obj.__members__:
            slots[name] = None
    if hasattr(obj, "__methods__"):
        for name in obj.__methods__:
            slots[name] = None
    # generic type
    if hasattr(obj, "__dict__"):
        slots.update(obj.__dict__)
    cls = type(obj)
    if hasattr(cls, "__mro__"):
        # new-style class, use descriptors
        for base in cls.__mro__:
            for name, value in base.__dict__.items():
                # XXX using callable() is a heuristic which isn't 100%
                # foolproof.
                if hasattr(value, "__get__") and not callable(value):
                    slots[name] = None
    if "__dict__" in slots:
        del slots["__dict__"]
    slots = slots.keys()
    slots.sort()
    return slots

def unpack_object(object, indent = 0):
    tp = type(object)
    if isinstance(object, SIMPLE_TYPES) and object is not None:
        raise TypeError, "can't browse simple type: %s" % tp.__name__
    elif isinstance(object, dict):
        return unpack_dict(object, indent)
    elif isinstance(object, (tuple, list)):
        return unpack_sequence(object, indent)
    elif isinstance(object, types.ModuleType):
        return unpack_dict(object.__dict__, indent)
    else:
        return unpack_other(object, indent)

def unpack_sequence(seq, indent = 0):
    return [(i, v, not isinstance(v, SIMPLE_TYPES), indent)
             for i, v in enumerate(seq)]

def unpack_dict(dict, indent = 0):
    items = dict.items()
    return pack_items(items, indent)

def unpack_instance(inst, indent = 0):
    if hasattr(inst, '__pybrowse_unpack__'):
        return unpack_object(inst.__pybrowse_unpack__(), indent)
    else:
        items = [('__class__', inst.__class__)] + inst.__dict__.items()
        return pack_items(items, indent)

def unpack_class(clss, indent = 0):
    items = [('__bases__', clss.__bases__), ('__name__', clss.__name__)] + clss.__dict__.items()
    return pack_items(items, indent)

def unpack_other(object, indent = 0):
    attrs = get_ivars(object)
    items = []
    for attr in attrs:
        try:
            value = getattr(object, attr)
        except:
            pass
        else:
            items.append((attr, value))
    return pack_items(items, indent)

def pack_items(items, indent = 0):
    items = [(k, v, not isinstance(v, SIMPLE_TYPES), indent)
             for k, v in items]
    return tuple_caselesssort(items)

def caselesssort(alist):
    """Return a sorted copy of a list. If there are only strings in the list,
    it will not consider case"""

    try:
        # turn ['FOO',  'aaBc', 'ABcD'] into [('foo', 'FOO'), ('aabc', 'aaBc'), ('abcd', 'ABcD')], if possible
        tupledlist = map(lambda item, lower = string.lower: (lower(item), item), alist)
    except TypeError:
        # at least one element in alist is not a string, proceed the normal way...
        alist = alist[:]
        alist.sort()
        return alist
    else:
        tupledlist.sort()
        # turn [('aabc', 'aaBc'), ('abcd', 'ABcD'), ('foo', 'FOO')] into ['aaBc', 'ABcD', 'FOO']
        return map(lambda x: x[1], tupledlist)

def tuple_caselesssort(items):
    try:
        tupledlist = map(lambda tuple, lower = string.lower: (lower(tuple[0]), tuple), items)
    except (AttributeError, TypeError):
        items = items[:]
        items.sort()
        return items
    else:
        tupledlist.sort()
        return map(lambda (low, tuple): tuple, tupledlist)



"""usage:
newsettings = FontDialog(('Chicago', 0, 12, (0, 0, 0))) # font name or id, style flags, size, color (color is ignored)
if newsettings:
        fontsettings, tabsettings = newsettings
        font, style, size, color = fontsettings # 'font' is always the font name, not the id number
        # do something
"""

import W
import PyEdit
from Carbon import TextEdit
from Carbon import Qd
import string
import types
import sys
import MacOS
if hasattr(MacOS, "SysBeep"):
    SysBeep = MacOS.SysBeep
else:
    def SysBeep(*args):
        pass

_stylenames = ["Plain", "Bold", "Italic", "Underline", "Outline", "Shadow", "Condensed", "Extended"]


class _FontDialog:

    #def __del__(self):
    #       print "doei!"

    def __init__(self, fontsettings, tabsettings):
        leftmargin = 60
        leftmargin2 = leftmargin - 16
        self.w = W.ModalDialog((440, 180), 'Font settings')
        self.w.fonttitle = W.TextBox((10, 12, leftmargin2, 14), "Font:", TextEdit.teJustRight)
        self.w.pop = W.FontMenu((leftmargin, 10, 16, 16), self.setfont)
        self.w.fontname = W.TextBox((leftmargin + 20, 12, 150, 14))
        self.w.sizetitle = W.TextBox((10, 38, leftmargin2, 14), "Size:", TextEdit.teJustRight)
        self.w.sizeedit = W.EditText((leftmargin, 35, 40, 20), "", self.checksize)
        styletop = 64
        self.w.styletitle = W.TextBox((10, styletop + 2, leftmargin2, 14), "Style:", TextEdit.teJustRight)
        for i in range(len(_stylenames)):
            top = styletop + (i % 4) * 20
            left = leftmargin + 80 * (i > 3) - 2
            if i:
                self.w[i] = W.CheckBox((left, top, 76, 16), _stylenames[i], self.dostyle)
            else:
                self.w[i] = W.CheckBox((left, top, 70, 16), _stylenames[i], self.doplain)

        if tabsettings:
            self.lasttab, self.tabmode = tabsettings
            self.w.tabsizetitle = W.TextBox((10, -26, leftmargin2, 14), "Tabsize:", TextEdit.teJustRight)
            self.w.tabsizeedit = W.EditText((leftmargin, -29, 40, 20), "", self.checktab)
            self.w.tabsizeedit.set(repr(self.lasttab))
            radiobuttons = []
            self.w.tabsizechars = W.RadioButton((leftmargin + 48, -26, 55, 14), "Spaces",
                            radiobuttons, self.toggletabmode)
            self.w.tabsizepixels = W.RadioButton((leftmargin + 110, -26, 55, 14), "Pixels",
                            radiobuttons, self.toggletabmode)
            if self.tabmode:
                self.w.tabsizechars.set(1)
            else:
                self.w.tabsizepixels.set(1)
        else:
            self.tabmode = None

        self.w.cancelbutton = W.Button((-180, -26, 80, 16), "Cancel", self.cancel)
        self.w.donebutton = W.Button((-90, -26, 80, 16), "Done", self.done)

        sampletext = "Sample text."
        self.w.sample = W.EditText((230, 10, -10, 130), sampletext,
                        fontsettings = fontsettings, tabsettings = tabsettings)

        self.w.setdefaultbutton(self.w.donebutton)
        self.w.bind('cmd.', self.w.cancelbutton.push)
        self.w.bind('cmdw', self.w.donebutton.push)
        self.lastsize = fontsettings[2]
        self._rv = None
        self.set(fontsettings)
        self.w.open()

    def toggletabmode(self, onoff):
        if self.w.tabsizechars.get():
            tabmode = 1
        else:
            tabmode = 0
        if self.tabmode <> tabmode:
            port = self.w.wid.GetWindowPort()
            (font, style, size, color), (tabsize, dummy) = self.get()
            savesettings = W.GetPortFontSettings(port)
            W.SetPortFontSettings(port, (font, style, size))
            spacewidth = Qd.StringWidth(' ')
            W.SetPortFontSettings(port, savesettings)
            if tabmode:
                # convert pixels to spaces
                self.lasttab = int(round(float(tabsize) / spacewidth))
            else:
                # convert spaces to pixels
                self.lasttab = spacewidth * tabsize
            self.w.tabsizeedit.set(repr(self.lasttab))
            self.tabmode = tabmode
            self.doit()

    def set(self, fontsettings):
        font, style, size, color = fontsettings
        if type(font) <> types.StringType:
            from Carbon import Res
            res = Res.GetResource('FOND', font)
            font = res.GetResInfo()[2]
        self.w.fontname.set(font)
        self.w.sizeedit.set(str(size))
        if style:
            for i in range(1, len(_stylenames)):
                self.w[i].set(style & 0x01)
                style = style >> 1
        else:
            self.w[0].set(1)

    def get(self):
        font = self.w.fontname.get()
        style = 0
        if not self.w[0].get():
            flag = 0x01
            for i in range(1, len(_stylenames)):
                if self.w[i].get():
                    style = style | flag
                flag = flag << 1
        size = self.lastsize
        if self.tabmode is None:
            return (font, style, size, (0, 0, 0)), (32, 0)
        else:
            return (font, style, size, (0, 0, 0)), (self.lasttab, self.tabmode)

    def doit(self):
        if self.w[0].get():
            style = 0
        else:
            style = 0
            for i in range(1, len(_stylenames)):
                if self.w[i].get():
                    style = style | 2 ** (i - 1)
        #self.w.sample.set(repr(style))
        fontsettings, tabsettings = self.get()
        self.w.sample.setfontsettings(fontsettings)
        self.w.sample.settabsettings(tabsettings)

    def checktab(self):
        tabsize = self.w.tabsizeedit.get()
        if not tabsize:
            return
        try:
            tabsize = string.atoi(tabsize)
        except (ValueError, OverflowError):
            good = 0
            sys.exc_traceback = None
        else:
            good = 1 <= tabsize <= 500
        if good:
            if self.lasttab <> tabsize:
                self.lasttab = tabsize
                self.doit()
        else:
            SysBeep(0)
            self.w.tabsizeedit.set(repr(self.lasttab))
            self.w.tabsizeedit.selectall()

    def checksize(self):
        size = self.w.sizeedit.get()
        if not size:
            return
        try:
            size = string.atoi(size)
        except (ValueError, OverflowError):
            good = 0
            sys.exc_traceback = None
        else:
            good = 1 <= size <= 500
        if good:
            if self.lastsize <> size:
                self.lastsize = size
                self.doit()
        else:
            SysBeep(0)
            self.w.sizeedit.set(repr(self.lastsize))
            self.w.sizeedit.selectall()

    def doplain(self):
        for i in range(1, len(_stylenames)):
            self.w[i].set(0)
        self.w[0].set(1)
        self.doit()

    def dostyle(self):
        for i in range(1, len(_stylenames)):
            if self.w[i].get():
                self.w[0].set(0)
                break
        else:
            self.w[0].set(1)
        self.doit()

    def close(self):
        self.w.close()
        del self.w

    def cancel(self):
        self.close()

    def done(self):
        self._rv = self.get()
        self.close()

    def setfont(self, fontname):
        self.w.fontname.set(fontname)
        self.doit()


def FontDialog(fontsettings, tabsettings = (32, 0)):
    fd = _FontDialog(fontsettings, tabsettings)
    return fd._rv

def test():
    print FontDialog(('Zapata-Light', 0, 25, (0, 0, 0)))



# Scan an Apple header file, generating a Python file of generator calls.
import sys
import os
from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)

from scantools import Scanner

def main():
    input = "QDOffscreen.h"
    output = "qdoffsgen.py"
    defsoutput = TOOLBOXDIR + "QDOffscreen.py"
    scanner = MyScanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()
    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "=== Done scanning and generating, now importing the generated code... ==="
    import qdoffssupport
    print "=== Done.  It's up to you to compile it now! ==="

class MyScanner(Scanner):

    def destination(self, type, name, arglist):
        classname = "Function"
        listname = "functions"
        if arglist:
            t, n, m = arglist[0]
            if t == "GWorldPtr" and m in ("InMode", "InOutMode"):
                classname = "Method"
                listname = "methods"
        return classname, listname

    def writeinitialdefs(self):
        self.defsfile.write("def FOUR_CHAR_CODE(x): return x\n")

    def makeblacklistnames(self):
        return [
                'DisposeGWorld', # Implied when the object is deleted
                'NewGWorldFromHBITMAP', # Don't know what the args do
                'GetGDeviceAttributes', # Windows-only
                ]

    def makeblacklisttypes(self):
        return [
                "void_ptr",             # GetGDeviceSurface, blacklisted for now
                "Ptr",                  # Again, for now (array is probably ok here)
                ]

    def makerepairinstructions(self):
        return [

##                      ("UpdateGWorld",
##                       [("GWorldPtr", "*", "OutMode")],
##                       [("*", "*", "InOutMode")]),

                # This one is incorrect: we say that all input gdevices are
                # optional, but some are not. Most are, however, so users passing
                # None for non-optional gdevices will get a qd error, I guess, in
                # stead of a python argument error.
                ([("GDHandle", "*", "InMode")],
                 [("OptGDHandle", "*", "InMode")]),
                ]

if __name__ == "__main__":
    main()



import unittest
from test import test_support

import os, socket
import StringIO

import urllib2
from urllib2 import Request, OpenerDirector

# XXX
# Request
# CacheFTPHandler (hard to write)
# parse_keqv_list, parse_http_list (I'm leaving this for Anthony Baxter
#  and Greg Stein, since they're doing Digest Authentication)
# Authentication stuff (ditto)
# ProxyHandler, CustomProxy, CustomProxyHandler (I don't use a proxy)
# GopherHandler (haven't used gopher for a decade or so...)

class TrivialTests(unittest.TestCase):
    def test_trivial(self):
        # A couple trivial tests

        self.assertRaises(ValueError, urllib2.urlopen, 'bogus url')

        # XXX Name hacking to get this to work on Windows.
        fname = os.path.abspath(urllib2.__file__).replace('\\', '/')
        if fname[1:2] == ":":
            fname = fname[2:]
        # And more hacking to get it to work on MacOS. This assumes
        # urllib.pathname2url works, unfortunately...
        if os.name == 'mac':
            fname = '/' + fname.replace(':', '/')
        elif os.name == 'riscos':
            import string
            fname = os.expand(fname)
            fname = fname.translate(string.maketrans("/.", "./"))

        file_url = "file://%s" % fname
        f = urllib2.urlopen(file_url)

        buf = f.read()
        f.close()


class MockOpener:
    addheaders = []
    def open(self, req, data=None):
        self.req, self.data = req, data
    def error(self, proto, *args):
        self.proto, self.args = proto, args

class MockFile:
    def read(self, count=None): pass
    def readline(self, count=None): pass
    def close(self): pass

class MockHeaders(dict):
    def getheaders(self, name):
        return self.values()

class MockResponse(StringIO.StringIO):
    def __init__(self, code, msg, headers, data, url=None):
        StringIO.StringIO.__init__(self, data)
        self.code, self.msg, self.headers, self.url = code, msg, headers, url
    def info(self):
        return self.headers
    def geturl(self):
        return self.url

class MockCookieJar:
    def add_cookie_header(self, request):
        self.ach_req = request
    def extract_cookies(self, response, request):
        self.ec_req, self.ec_r = request, response

class FakeMethod:
    def __init__(self, meth_name, action, handle):
        self.meth_name = meth_name
        self.handle = handle
        self.action = action
    def __call__(self, *args):
        return self.handle(self.meth_name, self.action, *args)

class MockHandler:
    def __init__(self, methods):
        self._define_methods(methods)
    def _define_methods(self, methods):
        for spec in methods:
            if len(spec) == 2: name, action = spec
            else: name, action = spec, None
            meth = FakeMethod(name, action, self.handle)
            setattr(self.__class__, name, meth)
    def handle(self, fn_name, action, *args, **kwds):
        self.parent.calls.append((self, fn_name, args, kwds))
        if action is None:
            return None
        elif action == "return self":
            return self
        elif action == "return response":
            res = MockResponse(200, "OK", {}, "")
            return res
        elif action == "return request":
            return Request("http://blah/")
        elif action.startswith("error"):
            code = action[action.rfind(" ")+1:]
            try:
                code = int(code)
            except ValueError:
                pass
            res = MockResponse(200, "OK", {}, "")
            return self.parent.error("http", args[0], res, code, "", {})
        elif action == "raise":
            raise urllib2.URLError("blah")
        assert False
    def close(self): pass
    def add_parent(self, parent):
        self.parent = parent
        self.parent.calls = []
    def __lt__(self, other):
        if not hasattr(other, "handler_order"):
            # No handler_order, leave in original order.  Yuck.
            return True
        return self.handler_order < other.handler_order

def add_ordered_mock_handlers(opener, meth_spec):
    """Create MockHandlers and add them to an OpenerDirector.

    meth_spec: list of lists of tuples and strings defining methods to define
    on handlers.  eg:

    [["http_error", "ftp_open"], ["http_open"]]

    defines methods .http_error() and .ftp_open() on one handler, and
    .http_open() on another.  These methods just record their arguments and
    return None.  Using a tuple instead of a string causes the method to
    perform some action (see MockHandler.handle()), eg:

    [["http_error"], [("http_open", "return request")]]

    defines .http_error() on one handler (which simply returns None), and
    .http_open() on another handler, which returns a Request object.

    """
    handlers = []
    count = 0
    for meths in meth_spec:
        class MockHandlerSubclass(MockHandler): pass
        h = MockHandlerSubclass(meths)
        h.handler_order = count
        h.add_parent(opener)
        count = count + 1
        handlers.append(h)
        opener.add_handler(h)
    return handlers

class OpenerDirectorTests(unittest.TestCase):

    def test_handled(self):
        # handler returning non-None means no more handlers will be called
        o = OpenerDirector()
        meth_spec = [
            ["http_open", "ftp_open", "http_error_302"],
            ["ftp_open"],
            [("http_open", "return self")],
            [("http_open", "return self")],
            ]
        handlers = add_ordered_mock_handlers(o, meth_spec)

        req = Request("http://example.com/")
        r = o.open(req)
        # Second .http_open() gets called, third doesn't, since second returned
        # non-None.  Handlers without .http_open() never get any methods called
        # on them.
        # In fact, second mock handler defining .http_open() returns self
        # (instead of response), which becomes the OpenerDirector's return
        # value.
        self.assertEqual(r, handlers[2])
        calls = [(handlers[0], "http_open"), (handlers[2], "http_open")]
        for expected, got in zip(calls, o.calls):
            handler, name, args, kwds = got
            self.assertEqual((handler, name), expected)
            self.assertEqual(args, (req,))

    def test_handler_order(self):
        o = OpenerDirector()
        handlers = []
        for meths, handler_order in [
            ([("http_open", "return self")], 500),
            (["http_open"], 0),
            ]:
            class MockHandlerSubclass(MockHandler): pass
            h = MockHandlerSubclass(meths)
            h.handler_order = handler_order
            handlers.append(h)
            o.add_handler(h)

        r = o.open("http://example.com/")
        # handlers called in reverse order, thanks to their sort order
        self.assertEqual(o.calls[0][0], handlers[1])
        self.assertEqual(o.calls[1][0], handlers[0])

    def test_raise(self):
        # raising URLError stops processing of request
        o = OpenerDirector()
        meth_spec = [
            [("http_open", "raise")],
            [("http_open", "return self")],
            ]
        handlers = add_ordered_mock_handlers(o, meth_spec)

        req = Request("http://example.com/")
        self.assertRaises(urllib2.URLError, o.open, req)
        self.assertEqual(o.calls, [(handlers[0], "http_open", (req,), {})])

##     def test_error(self):
##         # XXX this doesn't actually seem to be used in standard library,
##         #  but should really be tested anyway...

    def test_http_error(self):
        # XXX http_error_default
        # http errors are a special case
        o = OpenerDirector()
        meth_spec = [
            [("http_open", "error 302")],
            [("http_error_400", "raise"), "http_open"],
            [("http_error_302", "return response"), "http_error_303",
             "http_error"],
            [("http_error_302")],
            ]
        handlers = add_ordered_mock_handlers(o, meth_spec)

        class Unknown:
            def __eq__(self, other): return True

        req = Request("http://example.com/")
        r = o.open(req)
        assert len(o.calls) == 2
        calls = [(handlers[0], "http_open", (req,)),
                 (handlers[2], "http_error_302",
                  (req, Unknown(), 302, "", {}))]
        for expected, got in zip(calls, o.calls):
            handler, method_name, args = expected
            self.assertEqual((handler, method_name), got[:2])
            self.assertEqual(args, got[2])

    def test_processors(self):
        # *_request / *_response methods get called appropriately
        o = OpenerDirector()
        meth_spec = [
            [("http_request", "return request"),
             ("http_response", "return response")],
            [("http_request", "return request"),
             ("http_response", "return response")],
            ]
        handlers = add_ordered_mock_handlers(o, meth_spec)

        req = Request("http://example.com/")
        r = o.open(req)
        # processor methods are called on *all* handlers that define them,
        # not just the first handler that handles the request
        calls = [
            (handlers[0], "http_request"), (handlers[1], "http_request"),
            (handlers[0], "http_response"), (handlers[1], "http_response")]

        for i, (handler, name, args, kwds) in enumerate(o.calls):
            if i < 2:
                # *_request
                self.assertEqual((handler, name), calls[i])
                self.assertEqual(len(args), 1)
                self.assert_(isinstance(args[0], Request))
            else:
                # *_response
                self.assertEqual((handler, name), calls[i])
                self.assertEqual(len(args), 2)
                self.assert_(isinstance(args[0], Request))
                # response from opener.open is None, because there's no
                # handler that defines http_open to handle it
                self.assert_(args[1] is None or
                             isinstance(args[1], MockResponse))


def sanepathname2url(path):
    import urllib
    urlpath = urllib.pathname2url(path)
    if os.name == "nt" and urlpath.startswith("///"):
        urlpath = urlpath[2:]
    # XXX don't ask me about the mac...
    return urlpath

class HandlerTests(unittest.TestCase):

    def test_ftp(self):
        class MockFTPWrapper:
            def __init__(self, data): self.data = data
            def retrfile(self, filename, filetype):
                self.filename, self.filetype = filename, filetype
                return StringIO.StringIO(self.data), len(self.data)

        class NullFTPHandler(urllib2.FTPHandler):
            def __init__(self, data): self.data = data
            def connect_ftp(self, user, passwd, host, port, dirs):
                self.user, self.passwd = user, passwd
                self.host, self.port = host, port
                self.dirs = dirs
                self.ftpwrapper = MockFTPWrapper(self.data)
                return self.ftpwrapper

        import ftplib, socket
        data = "rheum rhaponicum"
        h = NullFTPHandler(data)
        o = h.parent = MockOpener()

        for url, host, port, type_, dirs, filename, mimetype in [
            ("ftp://localhost/foo/bar/baz.html",
             "localhost", ftplib.FTP_PORT, "I",
             ["foo", "bar"], "baz.html", "text/html"),
            ("ftp://localhost:80/foo/bar/",
             "localhost", 80, "D",
             ["foo", "bar"], "", None),
            ("ftp://localhost/baz.gif;type=a",
             "localhost", ftplib.FTP_PORT, "A",
             [], "baz.gif", None),  # XXX really this should guess image/gif
            ]:
            r = h.ftp_open(Request(url))
            # ftp authentication not yet implemented by FTPHandler
            self.assert_(h.user == h.passwd == "")
            self.assertEqual(h.host, socket.gethostbyname(host))
            self.assertEqual(h.port, port)
            self.assertEqual(h.dirs, dirs)
            self.assertEqual(h.ftpwrapper.filename, filename)
            self.assertEqual(h.ftpwrapper.filetype, type_)
            headers = r.info()
            self.assertEqual(headers.get("Content-type"), mimetype)
            self.assertEqual(int(headers["Content-length"]), len(data))

    def test_file(self):
        import time, rfc822, socket
        h = urllib2.FileHandler()
        o = h.parent = MockOpener()

        TESTFN = test_support.TESTFN
        urlpath = sanepathname2url(os.path.abspath(TESTFN))
        towrite = "hello, world\n"
        for url in [
            "file://localhost%s" % urlpath,
            "file://%s" % urlpath,
            "file://%s%s" % (socket.gethostbyname('localhost'), urlpath),
            "file://%s%s" % (socket.gethostbyname(socket.gethostname()),
                             urlpath),
            ]:
            f = open(TESTFN, "wb")
            try:
                try:
                    f.write(towrite)
                finally:
                    f.close()

                r = h.file_open(Request(url))
                try:
                    data = r.read()
                    headers = r.info()
                    newurl = r.geturl()
                finally:
                    r.close()
                stats = os.stat(TESTFN)
                modified = rfc822.formatdate(stats.st_mtime)
            finally:
                os.remove(TESTFN)
            self.assertEqual(data, towrite)
            self.assertEqual(headers["Content-type"], "text/plain")
            self.assertEqual(headers["Content-length"], "13")
            self.assertEqual(headers["Last-modified"], modified)

        for url in [
            "file://localhost:80%s" % urlpath,
# XXXX bug: these fail with socket.gaierror, should be URLError
##             "file://%s:80%s/%s" % (socket.gethostbyname('localhost'),
##                                    os.getcwd(), TESTFN),
##             "file://somerandomhost.ontheinternet.com%s/%s" %
##             (os.getcwd(), TESTFN),
            ]:
            try:
                f = open(TESTFN, "wb")
                try:
                    f.write(towrite)
                finally:
                    f.close()

                self.assertRaises(urllib2.URLError,
                                  h.file_open, Request(url))
            finally:
                os.remove(TESTFN)

        h = urllib2.FileHandler()
        o = h.parent = MockOpener()
        # XXXX why does // mean ftp (and /// mean not ftp!), and where
        #  is file: scheme specified?  I think this is really a bug, and
        #  what was intended was to distinguish between URLs like:
        # file:/blah.txt (a file)
        # file://localhost/blah.txt (a file)
        # file:///blah.txt (a file)
        # file://ftp.example.com/blah.txt (an ftp URL)
        for url, ftp in [
            ("file://ftp.example.com//foo.txt", True),
            ("file://ftp.example.com///foo.txt", False),
# XXXX bug: fails with OSError, should be URLError
            ("file://ftp.example.com/foo.txt", False),
            ]:
            req = Request(url)
            try:
                h.file_open(req)
            # XXXX remove OSError when bug fixed
            except (urllib2.URLError, OSError):
                self.assert_(not ftp)
            else:
                self.assert_(o.req is req)
                self.assertEqual(req.type, "ftp")

    def test_http(self):
        class MockHTTPResponse:
            def __init__(self, fp, msg, status, reason):
                self.fp = fp
                self.msg = msg
                self.status = status
                self.reason = reason
            def read(self):
                return ''
        class MockHTTPClass:
            def __init__(self):
                self.req_headers = []
                self.data = None
                self.raise_on_endheaders = False
            def __call__(self, host):
                self.host = host
                return self
            def set_debuglevel(self, level):
                self.level = level
            def request(self, method, url, body=None, headers={}):
                self.method = method
                self.selector = url
                self.req_headers += headers.items()
                if body:
                    self.data = body
                if self.raise_on_endheaders:
                    import socket
                    raise socket.error()
            def getresponse(self):
                return MockHTTPResponse(MockFile(), {}, 200, "OK")

        h = urllib2.AbstractHTTPHandler()
        o = h.parent = MockOpener()

        url = "http://example.com/"
        for method, data in [("GET", None), ("POST", "blah")]:
            req = Request(url, data, {"Foo": "bar"})
            req.add_unredirected_header("Spam", "eggs")
            http = MockHTTPClass()
            r = h.do_open(http, req)

            # result attributes
            r.read; r.readline  # wrapped MockFile methods
            r.info; r.geturl  # addinfourl methods
            r.code, r.msg == 200, "OK"  # added from MockHTTPClass.getreply()
            hdrs = r.info()
            hdrs.get; hdrs.has_key  # r.info() gives dict from .getreply()
            self.assertEqual(r.geturl(), url)

            self.assertEqual(http.host, "example.com")
            self.assertEqual(http.level, 0)
            self.assertEqual(http.method, method)
            self.assertEqual(http.selector, "/")
            self.assertEqual(http.req_headers,
                             [("Connection", "close"),
                              ("Foo", "bar"), ("Spam", "eggs")])
            self.assertEqual(http.data, data)

        # check socket.error converted to URLError
        http.raise_on_endheaders = True
        self.assertRaises(urllib2.URLError, h.do_open, http, req)

        # check adding of standard headers
        o.addheaders = [("Spam", "eggs")]
        for data in "", None:  # POST, GET
            req = Request("http://example.com/", data)
            r = MockResponse(200, "OK", {}, "")
            newreq = h.do_request_(req)
            if data is None:  # GET
                self.assert_("Content-length" not in req.unredirected_hdrs)
                self.assert_("Content-type" not in req.unredirected_hdrs)
            else:  # POST
                self.assertEqual(req.unredirected_hdrs["Content-length"], "0")
                self.assertEqual(req.unredirected_hdrs["Content-type"],
                             "application/x-www-form-urlencoded")
            # XXX the details of Host could be better tested
            self.assertEqual(req.unredirected_hdrs["Host"], "example.com")
            self.assertEqual(req.unredirected_hdrs["Spam"], "eggs")

            # don't clobber existing headers
            req.add_unredirected_header("Content-length", "foo")
            req.add_unredirected_header("Content-type", "bar")
            req.add_unredirected_header("Host", "baz")
            req.add_unredirected_header("Spam", "foo")
            newreq = h.do_request_(req)
            self.assertEqual(req.unredirected_hdrs["Content-length"], "foo")
            self.assertEqual(req.unredirected_hdrs["Content-type"], "bar")
            self.assertEqual(req.unredirected_hdrs["Host"], "baz")
            self.assertEqual(req.unredirected_hdrs["Spam"], "foo")

    def test_errors(self):
        h = urllib2.HTTPErrorProcessor()
        o = h.parent = MockOpener()

        url = "http://example.com/"
        req = Request(url)
        # 200 OK is passed through
        r = MockResponse(200, "OK", {}, "", url)
        newr = h.http_response(req, r)
        self.assert_(r is newr)
        self.assert_(not hasattr(o, "proto"))  # o.error not called
        # anything else calls o.error (and MockOpener returns None, here)
        r = MockResponse(201, "Created", {}, "", url)
        self.assert_(h.http_response(req, r) is None)
        self.assertEqual(o.proto, "http")  # o.error called
        self.assertEqual(o.args, (req, r, 201, "Created", {}))

    def test_cookies(self):
        cj = MockCookieJar()
        h = urllib2.HTTPCookieProcessor(cj)
        o = h.parent = MockOpener()

        req = Request("http://example.com/")
        r = MockResponse(200, "OK", {}, "")
        newreq = h.http_request(req)
        self.assert_(cj.ach_req is req is newreq)
        self.assertEquals(req.get_origin_req_host(), "example.com")
        self.assert_(not req.is_unverifiable())
        newr = h.http_response(req, r)
        self.assert_(cj.ec_req is req)
        self.assert_(cj.ec_r is r is newr)

    def test_redirect(self):
        from_url = "http://example.com/a.html"
        to_url = "http://example.com/b.html"
        h = urllib2.HTTPRedirectHandler()
        o = h.parent = MockOpener()

        # ordinary redirect behaviour
        for code in 301, 302, 303, 307:
            for data in None, "blah\nblah\n":
                method = getattr(h, "http_error_%s" % code)
                req = Request(from_url, data)
                req.add_header("Nonsense", "viking=withhold")
                req.add_unredirected_header("Spam", "spam")
                try:
                    method(req, MockFile(), code, "Blah",
                           MockHeaders({"location": to_url}))
                except urllib2.HTTPError:
                    # 307 in response to POST requires user OK
                    self.assert_(code == 307 and data is not None)
                self.assertEqual(o.req.get_full_url(), to_url)
                try:
                    self.assertEqual(o.req.get_method(), "GET")
                except AttributeError:
                    self.assert_(not o.req.has_data())
                self.assertEqual(o.req.headers["Nonsense"],
                                 "viking=withhold")
                self.assert_("Spam" not in o.req.headers)
                self.assert_("Spam" not in o.req.unredirected_hdrs)

        # loop detection
        req = Request(from_url)
        def redirect(h, req, url=to_url):
            h.http_error_302(req, MockFile(), 302, "Blah",
                             MockHeaders({"location": url}))
        # Note that the *original* request shares the same record of
        # redirections with the sub-requests caused by the redirections.

        # detect infinite loop redirect of a URL to itself
        req = Request(from_url, origin_req_host="example.com")
        count = 0
        try:
            while 1:
                redirect(h, req, "http://example.com/")
                count = count + 1
        except urllib2.HTTPError:
            # don't stop until max_repeats, because cookies may introduce state
            self.assertEqual(count, urllib2.HTTPRedirectHandler.max_repeats)

        # detect endless non-repeating chain of redirects
        req = Request(from_url, origin_req_host="example.com")
        count = 0
        try:
            while 1:
                redirect(h, req, "http://example.com/%d" % count)
                count = count + 1
        except urllib2.HTTPError:
            self.assertEqual(count,
                             urllib2.HTTPRedirectHandler.max_redirections)

    def test_cookie_redirect(self):
        class MockHTTPHandler(urllib2.HTTPHandler):
            def __init__(self): self._count = 0
            def http_open(self, req):
                import mimetools
                from StringIO import StringIO
                if self._count == 0:
                    self._count = self._count + 1
                    msg = mimetools.Message(
                        StringIO("Location: http://www.cracker.com/\r\n\r\n"))
                    return self.parent.error(
                        "http", req, MockFile(), 302, "Found", msg)
                else:
                    self.req = req
                    msg = mimetools.Message(StringIO("\r\n\r\n"))
                    return MockResponse(200, "OK", msg, "", req.get_full_url())
        # cookies shouldn't leak into redirected requests
        from cookielib import CookieJar
        from urllib2 import build_opener, HTTPHandler, HTTPError, \
             HTTPCookieProcessor

        from test_cookielib import interact_netscape

        cj = CookieJar()
        interact_netscape(cj, "http://www.example.com/", "spam=eggs")
        hh = MockHTTPHandler()
        cp = HTTPCookieProcessor(cj)
        o = build_opener(hh, cp)
        o.open("http://www.example.com/")
        self.assert_(not hh.req.has_header("Cookie"))


class MiscTests(unittest.TestCase):

    def test_build_opener(self):
        class MyHTTPHandler(urllib2.HTTPHandler): pass
        class FooHandler(urllib2.BaseHandler):
            def foo_open(self): pass
        class BarHandler(urllib2.BaseHandler):
            def bar_open(self): pass

        build_opener = urllib2.build_opener

        o = build_opener(FooHandler, BarHandler)
        self.opener_has_handler(o, FooHandler)
        self.opener_has_handler(o, BarHandler)

        # can take a mix of classes and instances
        o = build_opener(FooHandler, BarHandler())
        self.opener_has_handler(o, FooHandler)
        self.opener_has_handler(o, BarHandler)

        # subclasses of default handlers override default handlers
        o = build_opener(MyHTTPHandler)
        self.opener_has_handler(o, MyHTTPHandler)

        # a particular case of overriding: default handlers can be passed
        # in explicitly
        o = build_opener()
        self.opener_has_handler(o, urllib2.HTTPHandler)
        o = build_opener(urllib2.HTTPHandler)
        self.opener_has_handler(o, urllib2.HTTPHandler)
        o = build_opener(urllib2.HTTPHandler())
        self.opener_has_handler(o, urllib2.HTTPHandler)

    def opener_has_handler(self, opener, handler_class):
        for h in opener.handlers:
            if h.__class__ == handler_class:
                break
        else:
            self.assert_(False)

class NetworkTests(unittest.TestCase):
    def setUp(self):
        if 0:  # for debugging
            import logging
            logger = logging.getLogger("test_urllib2")
            logger.addHandler(logging.StreamHandler())

    def test_range (self):
        req = urllib2.Request("http://www.python.org",
                              headers={'Range': 'bytes=20-39'})
        result = urllib2.urlopen(req)
        data = result.read()
        self.assertEqual(len(data), 20)

    # XXX The rest of these tests aren't very good -- they don't check much.
    # They do sometimes catch some major disasters, though.

    def test_ftp(self):
        urls = [
            'ftp://www.python.org/pub/python/misc/sousa.au',
            'ftp://www.python.org/pub/tmp/blat',
            'ftp://gatekeeper.research.compaq.com/pub/DEC/SRC'
                '/research-reports/00README-Legal-Rules-Regs',
            ]
        self._test_urls(urls, self._extra_handlers())

    def test_gopher(self):
        urls = [
            # Thanks to Fred for finding these!
            'gopher://gopher.lib.ncsu.edu/11/library/stacks/Alex',
            'gopher://gopher.vt.edu:10010/10/33',
            ]
        self._test_urls(urls, self._extra_handlers())

    def test_file(self):
        TESTFN = test_support.TESTFN
        f = open(TESTFN, 'w')
        try:
            f.write('hi there\n')
            f.close()
            urls = [
                'file:'+sanepathname2url(os.path.abspath(TESTFN)),

                # XXX bug, should raise URLError
                #('file://nonsensename/etc/passwd', None, urllib2.URLError)
                ('file://nonsensename/etc/passwd', None, (OSError, socket.error))
                ]
            self._test_urls(urls, self._extra_handlers())
        finally:
            os.remove(TESTFN)

    def test_http(self):
        urls = [
            'http://www.espn.com/', # redirect
            'http://www.python.org/Spanish/Inquistion/',
            ('http://www.python.org/cgi-bin/faqw.py',
             'query=pythonistas&querytype=simple&casefold=yes&req=search', None),
            'http://www.python.org/',
            ]
        self._test_urls(urls, self._extra_handlers())

    # XXX Following test depends on machine configurations that are internal
    # to CNRI.  Need to set up a public server with the right authentication
    # configuration for test purposes.

##     def test_cnri(self):
##         if socket.gethostname() == 'bitdiddle':
##             localhost = 'bitdiddle.cnri.reston.va.us'
##         elif socket.gethostname() == 'bitdiddle.concentric.net':
##             localhost = 'localhost'
##         else:
##             localhost = None
##         if localhost is not None:
##             urls = [
##                 'file://%s/etc/passwd' % localhost,
##                 'http://%s/simple/' % localhost,
##                 'http://%s/digest/' % localhost,
##                 'http://%s/not/found.h' % localhost,
##                 ]

##             bauth = HTTPBasicAuthHandler()
##             bauth.add_password('basic_test_realm', localhost, 'jhylton',
##                                'password')
##             dauth = HTTPDigestAuthHandler()
##             dauth.add_password('digest_test_realm', localhost, 'jhylton',
##                                'password')

##             self._test_urls(urls, self._extra_handlers()+[bauth, dauth])

    def _test_urls(self, urls, handlers):
        import socket
        import time
        import logging
        debug = logging.getLogger("test_urllib2").debug

        urllib2.install_opener(urllib2.build_opener(*handlers))

        for url in urls:
            if isinstance(url, tuple):
                url, req, expected_err = url
            else:
                req = expected_err = None
            debug(url)
            try:
                f = urllib2.urlopen(url, req)
            except (IOError, socket.error, OSError), err:
                debug(err)
                if expected_err:
                    self.assert_(isinstance(err, expected_err))
            else:
                buf = f.read()
                f.close()
                debug("read %d bytes" % len(buf))
            debug("******** next url coming up...")
            time.sleep(0.1)

    def _extra_handlers(self):
        handlers = []

        handlers.append(urllib2.GopherHandler)

        cfh = urllib2.CacheFTPHandler()
        cfh.setTimeout(1)
        handlers.append(cfh)

##         # XXX try out some custom proxy objects too!
##         def at_cnri(req):
##             host = req.get_host()
##             debug(host)
##             if host[-18:] == '.cnri.reston.va.us':
##                 return True
##         p = CustomProxy('http', at_cnri, 'proxy.cnri.reston.va.us')
##         ph = CustomProxyHandler(p)
##         handlers.append(ph)

        return handlers


def test_main(verbose=None):
    tests = (TrivialTests,
             OpenerDirectorTests,
             HandlerTests,
             MiscTests)
    if test_support.is_resource_enabled('network'):
        tests += (NetworkTests,)
    test_support.run_unittest(*tests)

if __name__ == "__main__":
    test_main(verbose=True)



import compiler
import os
import test.test_support
import unittest

class CompilerTest(unittest.TestCase):

    def testCompileLibrary(self):
        # A simple but large test.  Compile all the code in the
        # standard library and its test suite.  This doesn't verify
        # that any of the code is correct, merely the compiler is able
        # to generate some kind of code for it.

        libdir = os.path.dirname(unittest.__file__)
        testdir = os.path.dirname(test.test_support.__file__)

        for dir in [libdir, testdir]:
            for basename in os.listdir(dir):
                if not basename.endswith(".py"):
                    continue
                path = os.path.join(dir, basename)
                if test.test_support.verbose:
                    print "compiling", path
                f = open(path)
                buf = f.read()
                f.close()
                if "badsyntax" in basename:
                    self.assertRaises(SyntaxError, compiler.compile,
                                      buf, basename, "exec")
                else:
                    compiler.compile(buf, basename, "exec")

def test_main():
    test.test_support.requires("compiler")
    test.test_support.run_unittest(CompilerTest)

if __name__ == "__main__":
    test_main()



import W
import Wkeys
from Carbon import Fm
import WASTEconst
from types import *
from Carbon import Events
import string
import sys
import traceback
import MacOS
import MacPrefs
from Carbon import Qd
import EasyDialogs
import PyInteractive

if not hasattr(sys, 'ps1'):
    sys.ps1 = '>>> '
if not hasattr(sys, 'ps2'):
    sys.ps2 = '... '

def inspect(foo):                       # JJS 1/25/99
    "Launch the browser on the given object.  This is a general built-in function."
    import PyBrowser
    PyBrowser.Browser(foo)

class ConsoleTextWidget(W.EditText):

    def __init__(self, *args, **kwargs):
        apply(W.EditText.__init__, (self,) + args, kwargs)
        self._inputstart = 0
        self._buf = ''
        self.pyinteractive = PyInteractive.PyInteractive()

        import __main__
        self._namespace = __main__.__dict__
        self._namespace['inspect'] = inspect                    # JJS 1/25/99

    def insert(self, text):
        self.checkselection()
        self.ted.WEInsert(text, None, None)
        self.changed = 1
        self.selchanged = 1

    def set_namespace(self, dict):
        if type(dict) <> DictionaryType:
            raise TypeError, "The namespace needs to be a dictionary"
        if 'inspect' not in dict.keys(): dict['inspect'] = inspect                      # JJS 1/25/99
        self._namespace = dict

    def open(self):
        import __main__
        W.EditText.open(self)
        self.write('Python %s\n' % sys.version)
        self.write('Type "copyright", "credits" or "license" for more information.\n')
        self.write('MacPython IDE %s\n' % __main__.__version__)
        self.write(sys.ps1)
        self.flush()

    def key(self, char, event):
        (what, message, when, where, modifiers) = event
        if self._enabled and not modifiers & Events.cmdKey or char in Wkeys.arrowkeys:
            if char not in Wkeys.navigationkeys:
                self.checkselection()
            if char == Wkeys.enterkey:
                char = Wkeys.returnkey
            selstart, selend = self.getselection()
            if char == Wkeys.backspacekey:
                if selstart <= (self._inputstart - (selstart <> selend)):
                    return
            self.ted.WEKey(ord(char), modifiers)
            if char not in Wkeys.navigationkeys:
                self.changed = 1
            if char not in Wkeys.scrollkeys:
                self.selchanged = 1
            self.updatescrollbars()
            if char == Wkeys.returnkey:
                text = self.get()[self._inputstart:selstart]
                text = string.join(string.split(text, "\r"), "\n")
                if hasattr(MacOS, 'EnableAppswitch'):
                    saveyield = MacOS.EnableAppswitch(0)
                self._scriptDone = False
                if sys.platform == "darwin":
                    # see identical construct in PyEdit.py
                    from threading import Thread
                    t = Thread(target=self._userCancelledMonitor,
                                    name="UserCancelledMonitor")
                    t.start()
                try:
                    self.pyinteractive.executeline(text, self, self._namespace)
                finally:
                    self._scriptDone = True
                if hasattr(MacOS, 'EnableAppswitch'):
                    MacOS.EnableAppswitch(saveyield)
                selstart, selend = self.getselection()
                self._inputstart = selstart

    def _userCancelledMonitor(self):
        # XXX duplicate code from PyEdit.py
        import time, os
        from signal import SIGINT
        from Carbon import Evt
        while not self._scriptDone:
            if Evt.CheckEventQueueForUserCancel():
                # Send a SIGINT signal to ourselves.
                # This gets delivered to the main thread,
                # cancelling the running script.
                os.kill(os.getpid(), SIGINT)
                break
            time.sleep(0.25)

    def domenu_save_as(self, *args):
        filename = EasyDialogs.AskFileForSave(message='Save console text as:',
                savedFileName='console.txt')
        if not filename:
            return
        f = open(filename, 'wb')
        f.write(self.get())
        f.close()
        MacOS.SetCreatorAndType(filename, W._signature, 'TEXT')

    def write(self, text):
        self._buf = self._buf + text
        if '\n' in self._buf:
            self.flush()

    def flush(self):
        stuff = string.split(self._buf, '\n')
        stuff = string.join(stuff, '\r')
        self.setselection_at_end()
        try:
            self.ted.WEInsert(stuff, None, None)
        finally:
            self._buf = ""
        selstart, selend = self.getselection()
        self._inputstart = selstart
        self.ted.WEClearUndo()
        self.updatescrollbars()
        if self._parentwindow.wid.GetWindowPort().QDIsPortBuffered():
            self._parentwindow.wid.GetWindowPort().QDFlushPortBuffer(None)

    def selection_ok(self):
        selstart, selend = self.getselection()
        return not (selstart < self._inputstart or selend < self._inputstart)

    def checkselection(self):
        if not self.selection_ok():
            self.setselection_at_end()

    def setselection_at_end(self):
        end = self.ted.WEGetTextLength()
        self.setselection(end, end)
        self.updatescrollbars()

    def domenu_cut(self, *args):
        if not self.selection_ok():
            return
        W.EditText.domenu_cut(self)

    def domenu_paste(self, *args):
        if not self.selection_ok():
            self.setselection_at_end()
        W.EditText.domenu_paste(self)

    def domenu_clear(self, *args):
        if not self.selection_ok():
            return
        W.EditText.domenu_clear(self)


class PyConsole(W.Window):

    def __init__(self, bounds, show = 1, fontsettings = ("Monaco", 0, 9, (0, 0, 0)),
                    tabsettings = (32, 0), unclosable = 0):
        W.Window.__init__(self,
                                bounds,
                                "Python Interactive",
                                minsize = (200, 100),
                                tabbable = 0,
                                show = show)

        self._unclosable = unclosable
        consoletext = ConsoleTextWidget((-1, -1, -14, 1), inset = (6, 5),
                        fontsettings = fontsettings, tabsettings = tabsettings)
        self._bary = W.Scrollbar((-15, 14, 16, -14), consoletext.vscroll, max = 32767)
        self.consoletext = consoletext
        self.namespacemenu = W.PopupMenu((-15, -1, 16, 16), [], self.consoletext.set_namespace)
        self.namespacemenu.bind('<click>', self.makenamespacemenu)
        self.open()

    def makenamespacemenu(self, *args):
        W.SetCursor('watch')
        namespacelist = self.getnamespacelist()
        self.namespacemenu.set([("Clear window", self.clearbuffer), ("Font settings\xc9", self.dofontsettings),
                        ["Namespace"] + namespacelist, ("Browse namespace\xc9", self.browsenamespace)])
        currentname = self.consoletext._namespace["__name__"]
        for i in range(len(namespacelist)):
            if namespacelist[i][0] == currentname:
                break
        else:
            return
        # XXX this functionality should be generally available in Wmenus
        submenuid = self.namespacemenu.menu.menu.GetItemMark(3)
        menu = self.namespacemenu.menu.bar.menus[submenuid]
        menu.menu.CheckMenuItem(i + 1, 1)

    def browsenamespace(self):
        import PyBrowser, W
        W.SetCursor('watch')
        PyBrowser.Browser(self.consoletext._namespace, self.consoletext._namespace["__name__"])

    def clearbuffer(self):
        from Carbon import Res
        self.consoletext.ted.WEUseText(Res.Resource(''))
        self.consoletext.write(sys.ps1)
        self.consoletext.flush()

    def getnamespacelist(self):
        import os
        import __main__
        editors = filter(lambda x: x.__class__.__name__ == "Editor", self.parent._windows.values())

        namespaces = [ ("__main__",__main__.__dict__) ]
        for ed in editors:
            modname = os.path.splitext(ed.title)[0]
            if sys.modules.has_key(modname):
                module = sys.modules[modname]
                namespaces.append((modname, module.__dict__))
            else:
                if ed.title[-3:] == '.py':
                    modname = ed.title[:-3]
                else:
                    modname = ed.title
                ed.globals["__name__"] = modname
                namespaces.append((modname, ed.globals))
        return namespaces

    def dofontsettings(self):
        import FontSettings
        settings = FontSettings.FontDialog(self.consoletext.getfontsettings(),
                        self.consoletext.gettabsettings())
        if settings:
            fontsettings, tabsettings = settings
            self.consoletext.setfontsettings(fontsettings)
            self.consoletext.settabsettings(tabsettings)

    def show(self, onoff = 1):
        W.Window.show(self, onoff)
        if onoff:
            self.select()

    def close(self):
        if self._unclosable:
            self.show(0)
            return -1
        W.Window.close(self)

    def writeprefs(self):
        prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
        prefs.console.show = self.isvisible()
        prefs.console.windowbounds = self.getbounds()
        prefs.console.fontsettings = self.consoletext.getfontsettings()
        prefs.console.tabsettings = self.consoletext.gettabsettings()
        prefs.save()

    def getselectedtext(self):
        return self.consoletext.getselectedtext()

class OutputTextWidget(W.EditText):

    def domenu_save_as(self, *args):
        title = self._parentwindow.gettitle()
        filename = EasyDialogs.AskFileForSave(message='Save %s text as:' % title,
                savedFileName=title + '.txt')
        if not filename:
            return
        f = open(filename, 'wb')
        f.write(self.get())
        f.close()
        MacOS.SetCreatorAndType(filename, W._signature, 'TEXT')

    def domenu_cut(self, *args):
        self.domenu_copy(*args)

    def domenu_clear(self, *args):
        self.set('')


class PyOutput:

    def __init__(self, bounds, show = 1, fontsettings = ("Monaco", 0, 9, (0, 0, 0)), tabsettings = (32, 0)):
        self.bounds = bounds
        self.fontsettings = fontsettings
        self.tabsettings = tabsettings
        self.w = None
        self.closed = 1
        self._buf = ''
        # should be able to set this
        self.savestdout, self.savestderr = sys.stdout, sys.stderr
        sys.stderr = sys.stdout = self
        if show:
            self.show()

    def setupwidgets(self):
        self.w = W.Window(self.bounds, "Output",
                        minsize = (200, 100),
                        tabbable = 0)
        self.w.outputtext = OutputTextWidget((-1, -1, -14, 1), inset = (6, 5),
                        fontsettings = self.fontsettings, tabsettings = self.tabsettings, readonly = 1)
        menuitems = [("Clear window", self.clearbuffer), ("Font settings\xc9", self.dofontsettings)]
        self.w.popupmenu = W.PopupMenu((-15, -1, 16, 16), menuitems)

        self.w._bary = W.Scrollbar((-15, 14, 16, -14), self.w.outputtext.vscroll, max = 32767)
        self.w.bind("<close>", self.close)
        self.w.bind("<activate>", self.activate)

    def write(self, text):
        if hasattr(MacOS, 'EnableAppswitch'):
            oldyield = MacOS.EnableAppswitch(-1)
        try:
            self._buf = self._buf + text
            if '\n' in self._buf:
                self.flush()
        finally:
            if hasattr(MacOS, 'EnableAppswitch'):
                MacOS.EnableAppswitch(oldyield)

    def flush(self):
        self.show()
        stuff = string.split(self._buf, '\n')
        stuff = string.join(stuff, '\r')
        end = self.w.outputtext.ted.WEGetTextLength()
        self.w.outputtext.setselection(end, end)
        self.w.outputtext.ted.WEFeatureFlag(WASTEconst.weFReadOnly, 0)
        try:
            self.w.outputtext.ted.WEInsert(stuff, None, None)
        finally:
            self._buf = ""
        self.w.outputtext.updatescrollbars()
        self.w.outputtext.ted.WEFeatureFlag(WASTEconst.weFReadOnly, 1)
        if self.w.wid.GetWindowPort().QDIsPortBuffered():
            self.w.wid.GetWindowPort().QDFlushPortBuffer(None)

    def show(self):
        if self.closed:
            if not self.w:
                self.setupwidgets()
                self.w.open()
                self.w.outputtext.updatescrollbars()
                self.closed = 0
            else:
                self.w.show(1)
                self.closed = 0
                self.w.select()

    def writeprefs(self):
        if self.w is not None:
            prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
            prefs.output.show = self.w.isvisible()
            prefs.output.windowbounds = self.w.getbounds()
            prefs.output.fontsettings = self.w.outputtext.getfontsettings()
            prefs.output.tabsettings = self.w.outputtext.gettabsettings()
            prefs.save()

    def dofontsettings(self):
        import FontSettings
        settings = FontSettings.FontDialog(self.w.outputtext.getfontsettings(),
                        self.w.outputtext.gettabsettings())
        if settings:
            fontsettings, tabsettings = settings
            self.w.outputtext.setfontsettings(fontsettings)
            self.w.outputtext.settabsettings(tabsettings)

    def clearbuffer(self):
        from Carbon import Res
        self.w.outputtext.set('')

    def activate(self, onoff):
        if onoff:
            self.closed = 0

    def close(self):
        self.w.show(0)
        self.closed = 1
        return -1


class SimpleStdin:

    def readline(self):
        import EasyDialogs
        # A trick to make the input dialog box a bit more palatable
        if hasattr(sys.stdout, '_buf'):
            prompt = sys.stdout._buf
        else:
            prompt = ""
        if not prompt:
            prompt = "Stdin input:"
        sys.stdout.flush()
        rv = EasyDialogs.AskString(prompt)
        if rv is None:
            return ""
        rv = rv + "\n"  # readline should include line terminator
        sys.stdout.write(rv)  # echo user's reply
        return rv


def installconsole(defaultshow = 1):
    global console
    prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
    if not prefs.console or not hasattr(prefs.console, 'show'):
        prefs.console.show = defaultshow
    if not hasattr(prefs.console, "windowbounds"):
        prefs.console.windowbounds = (450, 250)
    if not hasattr(prefs.console, "fontsettings"):
        prefs.console.fontsettings = ("Monaco", 0, 9, (0, 0, 0))
    if not hasattr(prefs.console, "tabsettings"):
        prefs.console.tabsettings = (32, 0)
    console = PyConsole(prefs.console.windowbounds, prefs.console.show,
                    prefs.console.fontsettings, prefs.console.tabsettings, 1)

def installoutput(defaultshow = 0, OutPutWindow = PyOutput):
    global output

    # quick 'n' dirty std in emulation
    sys.stdin = SimpleStdin()

    prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
    if not prefs.output or not hasattr(prefs.output, 'show'):
        prefs.output.show = defaultshow
    if not hasattr(prefs.output, "windowbounds"):
        prefs.output.windowbounds = (450, 250)
    if not hasattr(prefs.output, "fontsettings"):
        prefs.output.fontsettings = ("Monaco", 0, 9, (0, 0, 0))
    if not hasattr(prefs.output, "tabsettings"):
        prefs.output.tabsettings = (32, 0)
    output = OutPutWindow(prefs.output.windowbounds, prefs.output.show,
                    prefs.output.fontsettings, prefs.output.tabsettings)



"""fixversions - Fix version numbers in .plist files to match current
python version and date"""

import sys
import os
import time
import plistlib

SHORTVERSION = "%d.%d" % (sys.version_info[0], sys.version_info[1])
if sys.version_info[2]:
    SHORTVERSION = SHORTVERSION + ".%d" % sys.version_info[2]
if sys.version_info[3] != 'final':
    SHORTVERSION = SHORTVERSION + "%s%d" % (sys.version_info[3], sys.version_info[4])

COPYRIGHT = "(c) %d Python Software Foundation." % time.gmtime()[0]

LONGVERSION = SHORTVERSION + ", " + COPYRIGHT

def fix(file):
    plist = plistlib.Plist.fromFile(file)
    changed = False
    if plist.has_key("CFBundleGetInfoString") and \
                    plist["CFBundleGetInfoString"] != LONGVERSION:
        plist["CFBundleGetInfoString"] = LONGVERSION
        changed = True
    if plist.has_key("CFBundleLongVersionString") and \
                    plist["CFBundleLongVersionString"] != LONGVERSION:
        plist["CFBundleLongVersionString"] = LONGVERSION
        changed = True
    if plist.has_key("NSHumanReadableCopyright") and \
                    plist["NSHumanReadableCopyright"] != COPYRIGHT:
        plist["NSHumanReadableCopyright"] = COPYRIGHT
        changed = True
    if plist.has_key("CFBundleVersion") and \
                    plist["CFBundleVersion"] != SHORTVERSION:
        plist["CFBundleVersion"] = SHORTVERSION
        changed = True
    if plist.has_key("CFBundleShortVersionString") and \
                    plist["CFBundleShortVersionString"] != SHORTVERSION:
        plist["CFBundleShortVersionString"] = SHORTVERSION
        changed = True
    if changed:
        os.rename(file, file + '~')
        plist.write(file)

def main():
    if len(sys.argv) < 2:
        print "Usage: %s plistfile ..." % sys.argv[0]
        print "or: %s -a      fix standard Python plist files"
        sys.exit(1)
    if sys.argv[1] == "-a":
        files = [
                "../OSXResources/app/Info.plist",
                "../OSXResources/framework/version.plist",
                "../Tools/IDE/PackageManager.plist",
                "../Tools/IDE/PythonIDE.plist",
                "../scripts/BuildApplet.plist"
        ]
        if not os.path.exists(files[0]):
            print "%s -a must be run from Mac/OSX directory"
            sys.exit(1)
    else:
        files = sys.argv[1:]
    for file in files:
        fix(file)
    sys.exit(0)

if __name__ == "__main__":
    main()



"""macfreezegui - The GUI for macfreeze"""
from Carbon import Dlg
import macfs
import EasyDialogs
import sys
import os
import string
from Carbon import Res
import macresource

ID_MAINDIALOG=512

ITEM_SCRIPTNAME=2
ITEM_SCRIPTBROWSE=3
ITEM_GENSOURCE=4
ITEM_GENSOURCE_ITEMS=(7,)
ITEM_SOURCEDIRNAME=6
ITEM_SOURCEDIRBROWSE=7
ITEM_GENRESOURCE=8
ITEM_GENRESOURCE_ITEMS=(11,)
ITEM_RESOURCENAME=10
ITEM_RESOURCEBROWSE=11
ITEM_GENAPPLET=12
ITEM_GENAPPLET_ITEMS=(15,)
ITEM_APPLETNAME=14
ITEM_APPLETBROWSE=15
ITEM_OK=16
ITEM_CANCEL=17
ITEM_DEBUG=19
ITEM_GENINFO=20

RADIO_GROUPING={
        ITEM_GENSOURCE: ITEM_GENSOURCE_ITEMS,
        ITEM_GENRESOURCE: ITEM_GENRESOURCE_ITEMS,
        ITEM_GENAPPLET: ITEM_GENAPPLET_ITEMS,
        ITEM_GENINFO: ()
}

def dialog(script=None):

    # Invent the various names
    if not script:
        fss, ok = macfs.PromptGetFile("Script?", "TEXT")
        if not ok:
            sys.exit(0)
        script = fss.as_pathname()
    basename, ext = os.path.splitext(script)
    if ext:
        appletname = basename
        rsrcname = basename + 'modules.rsrc'
    else:
        appletname = script + '.applet'
        rsrcname = script + 'modules.rsrc'
    dirname, basebase = os.path.split(basename)
    dirname = os.path.join(dirname, 'build.'+basebase)

    # Get the dialog, possibly opening the resource file (if needed)
    macresource.need('DLOG', ID_MAINDIALOG, 'macfreeze.rsrc')
    d = Dlg.GetNewDialog(ID_MAINDIALOG, -1)
    if d == None:
        EasyDialogs.Message("Dialog resource not found or faulty")
        sys.exit(1)

    # Fill the dialog
    d.SetDialogDefaultItem(ITEM_OK)
    d.SetDialogCancelItem(ITEM_CANCEL)

    _dialogsetfile(d, ITEM_SCRIPTNAME, script)
    _dialogsetfile(d, ITEM_SOURCEDIRNAME, dirname)
    _dialogsetfile(d, ITEM_RESOURCENAME, rsrcname)
    _dialogsetfile(d, ITEM_APPLETNAME, appletname)

    gentype = ITEM_GENSOURCE
    _dialogradiogroup(d, ITEM_GENSOURCE)

    # Interact
    d.GetDialogWindow().SetWTitle("Standalone application creation options")
    d.GetDialogWindow().ShowWindow()
    d.DrawDialog()
    while 1:
        item = Dlg.ModalDialog(None)
        if item == ITEM_OK:
            break
        elif item == ITEM_CANCEL:
            sys.exit(0)
        elif item in RADIO_GROUPING.keys():
            gentype = item
            _dialogradiogroup(d, item)
        elif item == ITEM_SCRIPTBROWSE:
            fss, ok = macfs.PromptGetFile("Script?")
            if ok:
                script = fss.as_pathname()
                _dialogsetfile(d, ITEM_SCRIPTNAME, script)
        elif item == ITEM_SOURCEDIRBROWSE:
            fss, ok = macfs.StandardPutFile("Output folder name", os.path.split(dirname)[1])
            if ok:
                dirname = fss.as_pathname()
                _dialogsetfile(d, ITEM_SOURCEDIRNAME, dirname)
        elif item == ITEM_RESOURCEBROWSE:
            fss, ok = macfs.StandardPutFile("Resource output file", os.path.split(rsrcname)[1])
            if ok:
                rsrcname = fss.as_pathname()
                _dialogsetfile(d, ITEM_RESOURCENAME, rsrcname)
        elif item == ITEM_APPLETBROWSE:
            fss, ok = macfs.StandardPutFile("Applet output file", os.path.split(appletname)[1])
            if ok:
                appletname = fss.as_pathname()
                _dialogsetfile(d, ITEM_APPLETNAME, appletname)
        else:
            pass
    tp, h, rect = d.GetDialogItem(ITEM_DEBUG)
    debug = Dlg.GetDialogItemText(h)
    try:
        debug = string.atoi(string.strip(debug))
    except ValueError:
        EasyDialogs.Message("Illegal debug value %r, set to zero."%(debug,))
        debug = 0
    if gentype == ITEM_GENSOURCE:
        return 'source', script, dirname, debug
    elif gentype == ITEM_GENRESOURCE:
        return 'resource', script, rsrcname, debug
    elif gentype == ITEM_GENAPPLET:
        return 'applet', script, appletname, debug
    elif gentype == ITEM_GENINFO:
        return 'info', script, '', debug
    raise 'Error in gentype', gentype

def _dialogradiogroup(d, item):
    for k in RADIO_GROUPING.keys():
        subitems = RADIO_GROUPING[k]
        tp, h, rect = d.GetDialogItem(k)
        if k == item:
            h.as_Control().SetControlValue(1)
            for i2 in subitems:
                d.ShowDialogItem(i2)
        else:
            h.as_Control().SetControlValue(0)
            for i2 in subitems:
                d.HideDialogItem(i2)

def _dialogsetfile(d, item, file):
    if len(file) > 32:
        file = '\311:' + os.path.split(file)[1]
    tp, h, rect = d.GetDialogItem(item)
    Dlg.SetDialogItemText(h, file)

if __name__ == '__main__':
    type, script, file, debug = dialog()
    print type, script, file, 'debug=%d'%debug
    sys.exit(1)



# Scan an Apple header file, generating a Python file of generator calls.

import sys
import os
from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)
from scantools import Scanner

LONG = "Events"
SHORT = "evt"
OBJECT = "NOTUSED"

def main():
    input = LONG + ".h"
    output = SHORT + "gen.py"
    defsoutput = TOOLBOXDIR + LONG + ".py"
    scanner = MyScanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()
    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "=== Done scanning and generating, now importing the generated code... ==="
    exec "import " + SHORT + "support"
    print "=== Done.  It's up to you to compile it now! ==="

class MyScanner(Scanner):

    def destination(self, type, name, arglist):
        classname = "Function"
        listname = "functions"
        if arglist:
            t, n, m = arglist[0]
            # This is non-functional today
            if t == OBJECT and m == "InMode":
                classname = "Method"
                listname = "methods"
        return classname, listname

    def makeblacklistnames(self):
        return [
                "KeyTranslate",
                "GetEventMask",         # I cannot seem to find this routine...
                "WaitNextEvent",        # Manually generated because of optional region
                # Constants with funny definitions
                "osEvtMessageMask",
                # OS8 calls
                'SystemEvent',
                'SystemTask',
                'SystemClick',
                'GetOSEvent',
                'OSEventAvail',
                ]

    def makeblacklisttypes(self):
        return [
                "EvQElPtr", "QHdrPtr"
                ]

    def makerepairinstructions(self):
        return [
                ([("void_ptr", "*", "InMode"), ("long", "*", "InMode")],
                 [("InBuffer", "*", "*")]),

                ([("void", "*", "OutMode"), ("long", "*", "InMode"),
                                            ("long", "*", "OutMode")],
                 [("VarVarOutBuffer", "*", "InOutMode")]),

                ([("void", "wStorage", "OutMode")],
                 [("NullStorage", "*", "InMode")]),

                # GetKeys
                ([('KeyMap', 'theKeys', 'InMode')],
                 [('*', '*', 'OutMode')]),

                # GetTicker
                ([('unsigned long', '*', '*')],
                 [('unsigned_long', '*', '*')]),
                ]

if __name__ == "__main__":
    main()



import marshal
import types
from Carbon import Folder
from Carbon import Folders

class PrefObject:

    def __init__(self, dict = None):
        if dict == None:
            self._prefsdict = {}
        else:
            self._prefsdict = dict

    def __len__(self):
        return len(self._prefsdict)

    def __delattr__(self, attr):
        if self._prefsdict.has_key(attr):
            del self._prefsdict[attr]
        else:
            raise AttributeError, 'delete non-existing instance attribute'

    def __getattr__(self, attr):
        if attr == '__members__':
            keys = self._prefsdict.keys()
            keys.sort()
            return keys
        try:
            return self._prefsdict[attr]
        except KeyError:
            raise AttributeError, attr

    def __setattr__(self, attr, value):
        if attr[0] <> '_':
            self._prefsdict[attr] = value
        else:
            self.__dict__[attr] = value

    def getprefsdict(self):
        return self._prefsdict


class PrefFile(PrefObject):

    def __init__(self, path, creator = 'Pyth'):
        # Find the preferences folder and our prefs file, create if needed.
        self.__path = path
        self.__creator = creator
        self._prefsdict = {}
        try:
            prefdict = marshal.load(open(self.__path, 'rb'))
        except (IOError, ValueError):
            # file not found, or currupt marshal data
            pass
        else:
            for key, value in prefdict.items():
                if type(value) == types.DictType:
                    self._prefsdict[key] = PrefObject(value)
                else:
                    self._prefsdict[key] = value

    def save(self):
        prefdict = {}
        for key, value in self._prefsdict.items():
            if type(value) == types.InstanceType:
                prefdict[key] = value.getprefsdict()
                if not prefdict[key]:
                    del prefdict[key]
            else:
                prefdict[key] = value
        marshal.dump(prefdict, open(self.__path, 'wb'))
        try:
            MacOS.SetCreatorAndType(self.__path, self.__creator, 'pref')
        except:
            pass

    def __getattr__(self, attr):
        if attr == '__members__':
            keys = self._prefsdict.keys()
            keys.sort()
            return keys
        try:
            return self._prefsdict[attr]
        except KeyError:
            if attr[0] <> '_':
                self._prefsdict[attr] = PrefObject()
                return self._prefsdict[attr]
            else:
                raise AttributeError, attr


_prefscache = {}

def GetPrefs(prefname, creator = 'Pyth'):
    import macostools, os
    if _prefscache.has_key(prefname):
        return _prefscache[prefname]
    # Find the preferences folder and our prefs file, create if needed.
    fsr = Folder.FSFindFolder(Folders.kOnSystemDisk, 'pref', 1)
    prefsfolder = fsr.as_pathname()
    path = os.path.join(prefsfolder, prefname)
    head, tail = os.path.split(path)
    # make sure the folder(s) exist
    macostools.mkdirs(head)

    preffile = PrefFile(path, creator)
    _prefscache[prefname] = preffile
    return preffile



"""Parse sys/errno.h and Errors.h and create Estr resource"""

import re
import string
from Carbon import Res
import os

READ = 1
WRITE = 2
smAllScripts = -3

ERRNO_PROG="#define[ \t]+" \
                   "([A-Z0-9a-z_]+)" \
                   "[ \t]+" \
                   "([0-9]+)" \
                   "[ \t]*/\*[ \t]*" \
                   "(.*)" \
                   "[ \t]*\*/"

ERRORS_PROG="[ \t]*" \
                        "([A-Z0-9a-z_]+)" \
                        "[ \t]*=[ \t]*" \
                        "([-0-9]+)" \
                        "[, \t]*/\*[ \t]*" \
                        "(.*)" \
                        "[ \t]*\*/"

ERRORS_PROG_2="[ \t]*" \
                        "([A-Z0-9a-z_]+)" \
                        "[ \t]*=[ \t]*" \
                        "([-0-9]+)" \
                        "[, \t]*"

def Pstring(str):
    if len(str) > 255:
        raise ValueError, 'String too large'
    return chr(len(str))+str

def writeestr(dst, edict):
    """Create Estr resource file given a dictionary of errors."""

    os.unlink(dst.as_pathname())
    Res.FSpCreateResFile(dst, 'RSED', 'rsrc', smAllScripts)
    output = Res.FSpOpenResFile(dst, WRITE)
    Res.UseResFile(output)
    for num in edict.keys():
        res = Res.Resource(Pstring(edict[num][0]))
        res.AddResource('Estr', num, '')
        res.WriteResource()
    Res.CloseResFile(output)

def writepython(fp, dict):
    k = dict.keys()
    k.sort()
    for i in k:
        fp.write("%s\t=\t%d\t#%s\n"%(dict[i][1], i, dict[i][0]))


def parse_errno_h(fp, dict):
    errno_prog = re.compile(ERRNO_PROG)
    for line in fp.readlines():
        m = errno_prog.match(line)
        if m:
            number = string.atoi(m.group(2))
            name = m.group(1)
            desc = string.strip(m.group(3))

            if not dict.has_key(number):
                dict[number] = desc, name
            else:
                print 'DUPLICATE', number
                print '\t', dict[number]
                print '\t', (desc, name)

def parse_errors_h(fp, dict):
    errno_prog = re.compile(ERRORS_PROG)
    errno_prog_2 = re.compile(ERRORS_PROG_2)
    for line in fp.readlines():
        match = 0
        m = errno_prog.match(line)
        m2 = errno_prog_2.match(line)
        if m:
            number = string.atoi(m.group(2))
            name = m.group(1)
            desc = string.strip(m.group(3))
            match=1
        elif m2:
            number = string.atoi(m2.group(2))
            name = m2.group(1)
            desc = name
            match=1
        if match:
            if number > 0: continue

            if not dict.has_key(number):
                dict[number] = desc, name
            else:
                print 'DUPLICATE', number
                print '\t', dict[number]
                print '\t', (desc, name)
                if len(desc) > len(dict[number][0]):
                    print 'Pick second one'
                    dict[number] = desc, name

def main():
    dict = {}
    pathname = EasyDialogs.AskFileForOpen(message="Where is GUSI sys/errno.h?")
    if pathname:
        fp = open(pathname)
        parse_errno_h(fp, dict)
        fp.close()

    pathname = EasyDialogs.AskFileForOpen(message="Select cerrno (MSL) or cancel")
    if pathname:
        fp = open(pathname)
        parse_errno_h(fp, dict)
        fp.close()

    pathname = EasyDialogs.AskFileForOpen(message="Where is MacErrors.h?")
    if pathname:
        fp = open(pathname)
        parse_errors_h(fp, dict)
        fp.close()

    pathname = EasyDialogs.AskFileForOpen(message="Where is mkestrres-MacErrors.h?")
    if pathname:
        fp = open(pathname)
        parse_errors_h(fp, dict)
        fp.close()

    if not dict:
        return

    pathname = EasyDialogs.AskFileForSave(message="Resource output file?", savedFileName="errors.rsrc")
    if pathname:
        writeestr(fss, dict)

    pathname = EasyDialogs.AskFileForSave(message="Python output file?", savedFileName="macerrors.py")
    if pathname:
        fp = open(pathname, "w")
        writepython(fp, dict)
        fp.close()
        fss.SetCreatorType('Pyth', 'TEXT')

    pathname = EasyDialogs.AskFileForSave(message="Text output file?", savedFileName="errors.txt")
    if pathname:
        fp = open(pathname, "w")

        k = dict.keys()
        k.sort()
        for i in k:
            fp.write("%d\t%s\t%s\n"%(i, dict[i][1], dict[i][0]))
        fp.close()


if __name__ == '__main__':
    main()
